#include "globals.h"
#include "drive.h"
#include "NetMonBtn.h"
#include <stdio.h>

CDrive::CDrive(void)
{
	ZeroMemory(&m_layout, sizeof(m_layout));
}

CDrive::~CDrive(void)
{
	ClearLayout();
}

void CDrive::RecalcLayout(int height, BTN_PROPERTIES& m_properties)
{
	ClearLayout();

	m_layout.height = height;
	m_layout.width = 0;

	m_layout.labelWidth = 0;
	m_layout.spaceWidth = 0;
	m_layout.percentWidth = 0;
	m_layout.graphWidth = 0;

	LOGFONT lf;
	ZeroMemory(&lf, sizeof(lf));
	lf.lfHeight = m_layout.height * 3 / 4;
	lf.lfWeight = FW_BOLD;// FW_NORMAL;
	lf.lfCharSet = ANSI_CHARSET;
	lf.lfOutPrecision = OUT_CHARACTER_PRECIS;
	lf.lfClipPrecision = CLIP_DEFAULT_PRECIS;
	lf.lfQuality = ANTIALIASED_QUALITY;
	lf.lfPitchAndFamily = FF_DONTCARE;
	strcpy(lf.lfFaceName, m_properties.fontName);
	m_layout.fntLabel =  CreateFontIndirect(&lf);
	lf.lfWeight = FW_NORMAL;
	if(m_properties.textInOneLine)
	{
		lf.lfHeight = m_layout.height;
		m_layout.fntSpace = CreateFontIndirect(&lf);
	} else
	{
		int d = 0;
		if(m_properties.showFree) d++;
		if(m_properties.showUsed) d++;
		if(m_properties.showPercent && !m_properties.percentInsideGraph) d++;
		if(d)
		{
			lf.lfHeight = m_layout.height / d;
			m_layout.fntSpace = CreateFontIndirect(&lf);
		}
	}
	if(m_properties.showPercent)
	{
		if(m_properties.percentInsideGraph)
		{
			if(m_properties.graphType == GRAPH_ROW)
			{
				lf.lfHeight = m_layout.height - 2;
			} else
			{
				lf.lfHeight = m_layout.height / 2;
			}
			m_layout.fntPercent = CreateFontIndirect(&lf);
		}
	}

	HDC dc = GetDC(NULL);
	HFONT oldFont = (HFONT) SelectObject(dc, m_layout.fntLabel);
	
	SIZE sz;
	if(m_properties.showLabel)
	{
		GetTextExtentPoint32(dc, "G:", 2, &sz);
		m_layout.labelWidth = sz.cx + 2;
		m_layout.width += m_layout.labelWidth;
	}

	int textSize = 0;
	if(m_layout.fntSpace)
	{
		if(m_properties.textInOneLine)
		{
			SelectObject(dc, m_layout.fntSpace);
			GetTextExtentPoint32(dc, "00.00M", 6, &sz);
			m_layout.spaceWidth = sz.cx + 2;
			if(m_properties.showFree)
			{
				textSize += m_layout.spaceWidth;
			}
			if(m_properties.showUsed)
			{
				textSize += m_layout.spaceWidth;
			}
			if(m_properties.showPercent && !m_properties.percentInsideGraph)
			{
				GetTextExtentPoint32(dc, "000%", 4, &sz);
				m_layout.percentWidth = sz.cx + 2;
				textSize += m_layout.percentWidth;
			}
		} else
		{
			int max = 0;
			SelectObject(dc, m_layout.fntSpace);
			GetTextExtentPoint32(dc, "00.00M", 6, &sz);
			max = sz.cx + 2;
			if(m_properties.showPercent && !m_properties.percentInsideGraph)
			{
				GetTextExtentPoint32(dc, "000%", 4, &sz);
				if(max < sz.cx + 2)
				{
					max = sz.cx + 2;
				}
			}
			m_layout.spaceWidth = max;
			m_layout.percentWidth = max;
			if(m_properties.showFree || m_properties.showUsed || (m_properties.showPercent && !m_properties.percentInsideGraph))
			{
				textSize += m_layout.spaceWidth;
			}
		}
	}
	m_layout.width += textSize;

	SelectObject(dc, oldFont);
	ReleaseDC(NULL, dc);

	m_layout.graphWidth = 0;
	switch(m_properties.graphType)
	{
	case GRAPH_PIE:
		m_layout.graphWidth = m_layout.height;
		break;
	case GRAPH_ROW:
		m_layout.graphWidth = m_layout.height * 3;
		break;
	}
	m_layout.width += m_layout.graphWidth;
	m_layout.width += 3;
}

void CDrive::ClearLayout(void)
{
	if(m_layout.fntLabel)
	{
		DeleteObject(m_layout.fntLabel);
	}
	if(m_layout.fntPercent)
	{
		DeleteObject(m_layout.fntPercent);
	}
	if(m_layout.fntSpace)
	{
		DeleteObject(m_layout.fntSpace);
	}
	ZeroMemory(&m_layout, sizeof(m_layout));
}

void CDrive::DrawButton(HDC dc, int x, int y, BOOL isOverDrive, BTN_PROPERTIES& m_properties, DRIVE_SPACE& m_drives)
{
	int add = 0;
	HFONT fntOld = (HFONT) SelectObject(dc, m_layout.fntLabel);
	RECT rc;
	rc.top = y;
	rc.bottom = rc.top + m_layout.height;

	// Draw drive letter
	if(m_properties.showLabel)
	{
		rc.left = x + add;
		rc.right = rc.left + m_layout.labelWidth;
		add += m_layout.labelWidth;

		char drv[10];
		strcpy(drv, m_drives.drive);
		drv[2] = 0;

		DrawText(dc, drv, -1, &rc, DT_VCENTER | DT_CENTER);
	}

	BOOL isAlarm = FALSE;
	if(m_drives.alarmSizeType == AST_SIZE)
	{
		if(m_drives.freeSpace <= m_drives.alarmSize)
		{
			isAlarm = TRUE;
		}
	} else
	{
		double percent = (double) m_drives.freeSpace / (double) m_drives.totalSpace;
		if(percent <= m_drives.alarmPercent)
		{
			isAlarm = TRUE;
		}
	}
	
	// Draw graph and percent if need
	BOOL drawPercent = TRUE;
	switch(m_properties.graphType)
	{
	case GRAPH_ROW:
		{
			rc.left = x + add;
			rc.right = rc.left + m_layout.graphWidth;

			RECT rcUsed = rc;
			RECT rcFree = rc;
			rcUsed.right = rcUsed.left + (int)(((double)m_layout.graphWidth * (double)(m_drives.totalSpace - m_drives.freeSpace)) / (double) m_drives.totalSpace);
			rcFree.left = rcUsed.right;

			HBRUSH brUsed = NULL;
			HBRUSH brFree = NULL;

			if(!isAlarm)
			{
				brUsed = CreateSolidBrush(m_properties.usedColor);
				brFree = CreateSolidBrush(m_properties.freeColor);
			} else
			{
				brUsed = CreateSolidBrush(m_properties.usedColorAlarm);
				brFree = CreateSolidBrush(m_properties.freeColorAlarm);
			}

			rcUsed.top++;
			rcUsed.bottom--;
			FillRect(dc, &rcUsed, brUsed);
			rcFree.top++;
			rcFree.bottom--;
			FillRect(dc, &rcFree, brFree);

			DeleteObject(brUsed);
			DeleteObject(brFree);

			if(m_properties.showPercent && m_properties.percentInsideGraph)
			{
				int percent = 0;
				if(m_drives.totalSpace)
				{
					percent = (int) ((double)m_drives.freeSpace / (double)m_drives.totalSpace * 100.0);
				}

				char str[100];
				sprintf(str, "%d%%", percent);
				SelectObject(dc, m_layout.fntPercent);
				RECT rcText = rc;
				rcText.top++;
				rcText.bottom--;
				DrawText(dc, str, -1, &rcText, DT_VCENTER | DT_CENTER | DT_SINGLELINE);
				drawPercent = FALSE;
			}
			add += m_layout.graphWidth;
		}
		break;
	case GRAPH_PIE:
		{
			Graphics graphics(dc);

			rc.left = x + add;
			rc.right = rc.left + m_layout.graphWidth;

			float usedAngle = 360;
			float freeAngle = 0;
			if(m_drives.totalSpace)
			{
				freeAngle = 360 * (float)m_drives.freeSpace / (float)m_drives.totalSpace;
			}
			usedAngle = 360 - freeAngle;

			int nX = rc.left + (rc.right - rc.left) / 2; 
			int nY = rc.top + (rc.bottom - rc.top) / 2; 
			DWORD dwRadius = (rc.bottom - rc.top) / 2;
			
			float xStartAngle = 0; 

			Pen pen(Color(0));

			Color clrUsed;
			Color clrFree;

			if(!isAlarm)
			{
				clrUsed.SetValue(clrUsed.MakeARGB(255, GetRValue(m_properties.usedColor), GetGValue(m_properties.usedColor), GetBValue(m_properties.usedColor)));
				clrFree.SetValue(clrFree.MakeARGB(255, GetRValue(m_properties.freeColor), GetGValue(m_properties.freeColor), GetBValue(m_properties.freeColor)));
			} else
			{
				clrUsed.SetValue(clrUsed.MakeARGB(255, GetRValue(m_properties.usedColorAlarm), GetGValue(m_properties.usedColorAlarm), GetBValue(m_properties.usedColorAlarm)));
				clrFree.SetValue(clrFree.MakeARGB(255, GetRValue(m_properties.freeColorAlarm), GetGValue(m_properties.freeColorAlarm), GetBValue(m_properties.freeColorAlarm)));
			}


			SolidBrush brUsed(clrUsed);
			SolidBrush brFree(clrFree);
			
			graphics.SetSmoothingMode(SmoothingModeAntiAlias);
			Status st = graphics.FillPie(&brUsed, rc.left, rc.top, rc.bottom - rc.top, rc.bottom - rc.top, xStartAngle, usedAngle);
			xStartAngle += usedAngle;
			st = graphics.FillPie(&brFree, rc.left, rc.top, rc.bottom - rc.top, rc.bottom - rc.top, xStartAngle, freeAngle);

			if(m_properties.showPercent && m_properties.percentInsideGraph)
			{
				int percent = 0;
				if(m_drives.totalSpace)
				{
					percent = (int) ((double)m_drives.freeSpace / (double)m_drives.totalSpace * 100.0);
				}

				char str[100];
				sprintf(str, "%d%%", percent);
				SelectObject(dc, m_layout.fntPercent);
				DrawText(dc, str, -1, &rc, DT_VCENTER | DT_CENTER | DT_SINGLELINE);
				drawPercent = FALSE;
			}
			add += m_layout.graphWidth;
		}
		break;
	}

	//Draw info text
	if(m_properties.textInOneLine)
	{
		if(m_properties.showFree)
		{
			rc.left = x + add;
			rc.right = rc.left + m_layout.spaceWidth;
			add += m_layout.spaceWidth;
			
			char str[200];
			Space2String(m_drives.freeSpace, str);
			SelectObject(dc, m_layout.fntSpace);
			DrawText(dc, str, -1, &rc, DT_VCENTER | DT_RIGHT);
		}
		if(m_properties.showUsed)
		{
			rc.left = x + add;
			rc.right = rc.left + m_layout.spaceWidth;
			add += m_layout.spaceWidth;
			
			char str[200];
			Space2String(m_drives.totalSpace, str);
			SelectObject(dc, m_layout.fntSpace);
			DrawText(dc, str, -1, &rc, DT_VCENTER | DT_RIGHT);
		}
		if(m_properties.showPercent && drawPercent)
		{
			rc.left = x + add;
			rc.right = rc.left + m_layout.percentWidth;
			add += m_layout.spaceWidth;

			int percent = 0;
			if(m_drives.totalSpace)
			{
				percent = (int) ((double)m_drives.freeSpace / (double)m_drives.totalSpace * 100.0);
			}

			char str[200];
			sprintf(str, "%d%%", percent);
			SelectObject(dc, m_layout.fntSpace);
			DrawText(dc, str, -1, &rc, DT_VCENTER | DT_RIGHT);
		}
	} else
	{
		int d = 0;
		if(m_properties.showFree) d++;
		if(m_properties.showUsed) d++;
		if(m_properties.showPercent && !m_properties.percentInsideGraph) d++;
		if(d)
		{
			rc.left = x + add;
			rc.right = rc.left + m_layout.spaceWidth;
			add += m_layout.spaceWidth;
			RECT rcText = rc;
			if(m_properties.showFree)
			{
				rcText.bottom = rcText.top + m_layout.height / d;
				char str[200];
				Space2String(m_drives.freeSpace, str);
				SelectObject(dc, m_layout.fntSpace);
				DrawText(dc, str, -1, &rcText, DT_VCENTER | DT_RIGHT);
				rcText.top = rcText.bottom;
			}
			if(m_properties.showUsed)
			{
				rcText.bottom = rcText.top + m_layout.height / d;
				char str[200];
				Space2String(m_drives.totalSpace, str);
				SelectObject(dc, m_layout.fntSpace);
				DrawText(dc, str, -1, &rcText, DT_VCENTER | DT_RIGHT);
				rcText.top = rcText.bottom;
			}
			if(m_properties.showPercent && drawPercent)
			{
				rcText.bottom = rcText.top + m_layout.height / d;
				int percent = 0;
				if(m_drives.totalSpace)
				{
					percent = (int) ((double)m_drives.freeSpace / (double)m_drives.totalSpace * 100.0);
				}

				char str[200];
				sprintf(str, "%d%%", percent);
				SelectObject(dc, m_layout.fntSpace);
				DrawText(dc, str, -1, &rcText, DT_VCENTER | DT_RIGHT);
				rcText.top = rcText.bottom;
			}
		}
	}
	SelectObject(dc, fntOld);

	if(isOverDrive)
	{
		Graphics graphics(dc);
		SolidBrush br(Color(100, GetRValue(m_properties.selectColor), GetGValue(m_properties.selectColor), GetBValue(m_properties.selectColor)));
		Rect rcDrive(x, y, m_layout.width, m_layout.height);
		graphics.FillRectangle(&br, rcDrive);
	}
}

void CDrive::Space2String(__int64 space, char* str)
{
	double Space = (double)space / (1024.0 * 1024.0);
	if(Space >= 1000)
	{
		Space /= 1024.0;
		sprintf(str, "%.2fG", Space);
	} else
	{
		sprintf(str, "%.0fM", Space);
	}
}
