// NetMonBtn.cpp: implementation of the CNetMonBtn class.
//
//////////////////////////////////////////////////////////////////////

#include "globals.h"
#include "NetMonBtn.h"
#include <shellapi.h>
#include <Shlwapi.h>
#include <stdio.h>
#include "PropertiesDlg.h"
#include "resource.h"

BOOL FindConnectionName(LPWSTR id, LPSTR name, ULONG cbSize)
{
	char chID[500];
	WideCharToMultiByte(CP_ACP, 0, id, -1, chID, 500, 0, 0);
	
	char* guidBegin = strstr(chID, "{");
	char* guidEnd =   strstr(chID, "}");
	if(!guidBegin || !guidEnd) return FALSE;
	char* guid = new char[guidEnd - guidBegin + 2];
	strncpy(guid, guidBegin, guidEnd - guidBegin + 1);
	guid[guidEnd - guidBegin + 1] = 0;
	char regKey[500] = "SYSTEM\\CurrentControlSet\\Control\\Network\\{4D36E972-E325-11CE-BFC1-08002BE10318}\\";
    strcat(regKey, guid);
    strcat(regKey, "\\");
    strcat(regKey, "Connection");

	BOOL ret = FALSE;
	HKEY key = NULL;
	if(RegOpenKey(HKEY_LOCAL_MACHINE, regKey, &key) == ERROR_SUCCESS)
	{
		DWORD sz = cbSize;
		if(RegQueryValueEx(key, "Name", 0, NULL, (LPBYTE) name, &sz) == ERROR_SUCCESS)
		{
			ret = TRUE;
		}
		RegCloseKey(key);
	}
	delete guid;

	return ret;
}

void GetFullSkinPath(LPSTR skinPath, LPSTR fullSkinPath)
{
	GetModuleFileName(g_hInst, fullSkinPath, MAX_PATH);
	PathRemoveFileSpec(fullSkinPath);
	PathAddBackslash(fullSkinPath);
	strcat(fullSkinPath, "skins\\");
	strcat(fullSkinPath, skinPath);
}

void GetRelSkinPath(LPSTR fullSkinPath, LPSTR skinPath)
{
	char* str = StrStrI(fullSkinPath, "\\skins\\");
	if(str)
	{
		str += strlen("\\skins\\");
		strcpy(skinPath, str);
	}
}

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

FIELDS_NAMES fieldsNames[MAX_FIELDS] = 
{
	{FIELDS_SPEED,			STR_ID_SPEED},
	{FIELDS_RECEIVED,		STR_ID_RECEIVED},
	{FIELDS_SEND,			STR_ID_SEND},
	{FIELDS_TIME,			STR_ID_TIME},
	{FIELDS_INBANDWIDTH,	STR_ID_INBANDWIDTH},
	{FIELDS_OUTBANDWIDTH,	STR_ID_OUTBANDWIDTH},
	{FIELDS_BANDWIDTH,		STR_ID_BANDWIDTH},
	{FIELDS_INLOAD,			STR_ID_INLOAD},
	{FIELDS_OUTLOAD,		STR_ID_OUTLOAD},
	{FIELDS_LOAD,			STR_ID_LOAD},
	{FIELDS_IP,				STR_ID_IP},
	{FIELDS_NAME,			STR_ID_NAME}
};

CNetMonBtn::CNetMonBtn()
{

	m_icModem = (HICON) LoadImage(g_hInst, MAKEINTRESOURCE(IDI_MODEM), IMAGE_ICON, 32, 32, LR_DEFAULTCOLOR);
	m_icNetcard = (HICON) LoadImage(g_hInst, MAKEINTRESOURCE(IDI_NETCARD), IMAGE_ICON, 32, 32, LR_DEFAULTCOLOR);

	m_propDlg = NULL;

	strcpy(m_run, "control netconnections");

	m_onConnectSound[0] = 0;
	m_onDisconnectSound[0] = 0;

	m_rows = 2;
	m_cols = 1;

	m_hResource = g_hInst;
	m_state = 0;
	m_ID = 0;
	m_version = 3;
	m_container = NULL;
	hStop = CreateEvent(NULL, FALSE, FALSE, NULL);
	hTimerThread = NULL;

	ZeroMemory(&m_layout, sizeof(m_layout));
	ZeroMemory(&m_properties, sizeof(BTN_PROPERTIES));
	m_properties.rows = 1;

	for(int i=0; i < MAX_FIELDS; i++)
	{
		m_params[i].paramID = fieldsNames[i].fieldID;
		m_params[i].show = FALSE;
	}
	m_params[1].show = TRUE;
	m_params[2].show = TRUE;
	m_params[6].show = TRUE;

	strcpy(m_properties.fontName, "Verdana");

	m_skinPath[0] = 0;

	InitializeCriticalSection(&m_sync);

	m_ObjRefCount = 1;
	g_DllRefCount++;
}

CNetMonBtn::~CNetMonBtn()
{
	DestroyIcon(m_icModem);
	DestroyIcon(m_icNetcard);
	if(m_propDlg)
	{
		m_propDlg->Close();
	}
	if(hTimerThread)
	{
		SetEvent(hStop);
		WaitForSingleObject(hTimerThread, INFINITE);
		CloseHandle(hTimerThread);
	}
	CloseHandle(hStop);

	DeleteCriticalSection(&m_sync);

	g_DllRefCount--;
}

//////////////////////////////////////////////////////////////////////
// IUnknown methods
//////////////////////////////////////////////////////////////////////

STDMETHODIMP CNetMonBtn::QueryInterface(REFIID riid, LPVOID *ppReturn)
{
	*ppReturn = NULL;

	//IUnknown
	if(IsEqualIID(riid, IID_ITlbButton))
	   {
	   *ppReturn = (IDataObject*) this;
	   }

	if(IsEqualIID(riid, IID_IUnknown))
	   {
	   *ppReturn = (IUnknown*) this;
	   }

	if(*ppReturn)
	   {
	   (*(LPUNKNOWN*)ppReturn)->AddRef();
	   return S_OK;
	   }

	return E_NOINTERFACE;
}

STDMETHODIMP_(DWORD) CNetMonBtn::AddRef()
{
	return ++m_ObjRefCount;
}

STDMETHODIMP_(DWORD) CNetMonBtn::Release()
{
	if(--m_ObjRefCount == 0)
	{
		delete this;
		return 0;
	}
	return m_ObjRefCount;
}

//////////////////////////////////////////////////////////////////////
// ITlbButton methods
//////////////////////////////////////////////////////////////////////


// Load parameters for button
STDMETHODIMP CNetMonBtn::Load(IStream *data)
{
	DWORD loadedVersion = 0;
	ULONG cbRead = 0;
	data->Read(&loadedVersion, sizeof(DWORD), &cbRead);

	DWORD sz = 0;
	data->Read(&sz, sizeof(sz), &cbRead);
	data->Read(&m_properties, sz, &cbRead);

	data->Read(&sz, sizeof(sz), &cbRead);
	data->Read(m_params, sz, &cbRead);

	data->Read(&sz, sizeof(sz), &cbRead);
	data->Read(m_skinPath, sz, &cbRead);

	m_skin.Load(m_skinPath, m_params);

	data->Read(m_onConnectSound, sizeof(m_onConnectSound), &cbRead);
	data->Read(m_onDisconnectSound, sizeof(m_onDisconnectSound), &cbRead);

	if(loadedVersion >= 2)
	{
		DWORD n = 0;
		data->Read(&n, sizeof(n), &cbRead);
		DWORD arrSz = sizeof(CUSTOM_CONNECTION);
		data->Read(&arrSz, sizeof(arrSz), &cbRead);
		void* buf = malloc(arrSz);
		for(DWORD i=0; i < n; i++)
		{
			data->Read(buf, arrSz, &cbRead);
			CUSTOM_CONNECTION con;
			ZeroMemory(&con, sizeof(con));
			memcpy(&con, buf, min(sizeof(con), arrSz));
			m_customConnections.Add(&con);
		}
		free(buf);
		if(loadedVersion >= 3)
		{
			data->Read(m_run, sizeof(m_run), &cbRead);
		}
	}

	StartTimer();
	return S_OK;
}

// Save button parameters
STDMETHODIMP CNetMonBtn::Save(IStream *data)
{
	ULONG cbWritten;
	data->Write(&m_version, sizeof(DWORD), &cbWritten);

	DWORD sz = sizeof(m_properties);
	data->Write(&sz, sizeof(sz), &cbWritten);
	data->Write(&m_properties, sizeof(m_properties), &cbWritten);

	sz = sizeof(m_params);
	data->Write(&sz, sizeof(sz), &cbWritten);
	data->Write(m_params, sizeof(m_params), &cbWritten);

	sz = strlen(m_skinPath) + 1;
	data->Write(&sz, sizeof(sz), &cbWritten);
	data->Write(m_skinPath, sz, &cbWritten);

	data->Write(m_onConnectSound, sizeof(m_onConnectSound), &cbWritten);
	data->Write(m_onDisconnectSound, sizeof(m_onDisconnectSound), &cbWritten);

	sz = m_customConnections.GetCount();
	data->Write(&sz, sizeof(sz), &cbWritten);
	sz = sizeof(CUSTOM_CONNECTION);
	data->Write(&sz, sizeof(sz), &cbWritten);
	for(int i=0; i < m_customConnections.GetCount(); i++)
	{
		data->Write(&m_customConnections[i], sz, &cbWritten);
	}

	data->Write(m_run, sizeof(m_run), &cbWritten);

	return S_OK;
}

// Sets button rectangle
STDMETHODIMP CNetMonBtn::SetRect(RECT *rcItem)
{
	m_rc = rcItem[0];

	m_layout.x = (m_rc.right - m_rc.left) / 2 - (m_layout.cols * m_layout.itemWidth) / 2;
	if(m_layout.x < 0) m_layout.x = 0;

	m_layout.y = (m_rc.bottom - m_rc.top) / 2 - (m_layout.rows * m_layout.itemHeight) / 2;
	if(m_layout.y < 0) m_layout.y = 0;

	return S_OK;
}



// Set button container
// NOTE: do not call AddRef to store container pointer
STDMETHODIMP CNetMonBtn::SetContainer(ITlbContainer *cont)
{
	m_container = cont;
	return S_OK;
}

STDMETHODIMP CNetMonBtn::SetID(int btnID)
{
	m_ID = btnID;
	return S_OK;
}

STDMETHODIMP CNetMonBtn::Draw(HANDLE hDC, RECT *rcItem)
{
	HDC dc = (HDC) hDC;

	COLORREF color = 0;
	RECT marging;
	if(m_container)
	{
		m_container->GetTextProps(&marging, &color);
	}

	CONTAINER_PARAMS cParams = {0, 0, 0};
	cParams.cbSize = sizeof(CONTAINER_PARAMS);
	int szIcon = 16;
	int isMenu = FALSE;
	if(m_container)
	{
		m_container->GetParams(&cParams);
		if(cParams.style & CTS_ISMENU)
		{
			isMenu = TRUE;
		}
		szIcon = cParams.dwIconSize;
	}

	SetTextColor(dc, color);
	SetBkMode(dc, TRANSPARENT);

	if(!m_connections.GetCount())
	{
		int iconLeft = 0;
		int iconTop = 0;

		if(m_container)
		{
			POINT pt;
			m_container->GetIconPoint(&pt);
			iconLeft = rcItem->left + pt.x;
			iconTop = rcItem->top + pt.y;
		}
		HICON hIcon = (HICON) LoadImage(g_hInst, MAKEINTRESOURCE(IDI_DEFAULT), IMAGE_ICON, szIcon, szIcon, LR_DEFAULTCOLOR);
		
		DrawIconEx(dc, iconLeft, iconTop, hIcon, szIcon, szIcon, 0, 0, DI_NORMAL);
		DestroyIcon(hIcon);
	} else
	{
		int x = rcItem->left + m_layout.x;
		int i=0;
		LockItems();
		for(int col=0; col < m_layout.cols; col++)
		{
			int y = rcItem->top + m_layout.y;
			if(i >= m_connections.GetCount()) break;
			for(int row = 0; row < m_layout.rows; row++)
			{
				if(i >= m_connections.GetCount()) break;

				m_skin.Draw(dc, x, y, &m_connections[i]);

				y += m_layout.itemHeight;
				i++;
			}
			x += m_layout.itemWidth;
		}
		UnlockItems();
	}
	return S_OK;
}

STDMETHODIMP CNetMonBtn::OnMouseOver(long keys, int x, int y)
{
/*	int overDrive = FindDriveXY(x, y);
	if(m_overDrive != overDrive)
	{
		m_overDrive = overDrive;
		m_container->UpdateButtonInfo(UPDATE_REDRAW);
	}*/
	return S_OK;
}

STDMETHODIMP CNetMonBtn::OnMouseLeave(long keys)
{
/*	if(m_overDrive >= 0)
	{
		m_overDrive = -1;
		m_container->UpdateButtonInfo(UPDATE_REDRAW);
	}*/
	return S_OK;
}

STDMETHODIMP CNetMonBtn::OnMouseEnter(long keys, int x, int y)
{
	return S_FALSE;
}

STDMETHODIMP CNetMonBtn::Create(IStream* data)
{
	if(m_container)
	{
		HWND hWndBand = m_container->GetBandWND();
		if(!m_propDlg)
		{
			m_propDlg = new CPropertiesPP;
			//CPropertiesPP dlg;
			m_propDlg->m_props = m_properties;
			memcpy(m_propDlg->m_params, m_params, sizeof(m_params));
			strcpy(m_propDlg->m_skinPath, m_skinPath);
			strcpy(m_propDlg->m_onConnectSound, m_onConnectSound);
			strcpy(m_propDlg->m_onDisconnectSound, m_onDisconnectSound);
			strcpy(m_propDlg->m_run, m_run);
			m_propDlg->m_hResource = m_hResource;
			m_propDlg->m_customConnections = m_customConnections;
			if(m_propDlg->Open(hWndBand, m_hResource) == IDOK)
			{
				m_customConnections = m_propDlg->m_customConnections;
				LoadConnections();

				m_properties = m_propDlg->m_props;
				strcpy(m_skinPath, m_propDlg->m_skinPath);
				strcpy(m_run, m_propDlg->m_run);
				strcpy(m_onConnectSound, m_propDlg->m_onConnectSound);
				strcpy(m_onDisconnectSound, m_propDlg->m_onDisconnectSound);
				memcpy(m_params, m_propDlg->m_params, sizeof(m_params));
				//m_container->UpdateButtonInfo(UPDATE_SIZE | UPDATE_REDRAW);
				delete m_propDlg;
				m_propDlg = NULL;
				return Save(data);
			}
			delete m_propDlg;
			m_propDlg = NULL;
		} else
		{
			m_propDlg->Open(NULL, NULL);
		}
	}
	return S_FALSE;
}


STDMETHODIMP CNetMonBtn::StateChanged(int state, int* needRedraw)
{
	needRedraw[0] = TRUE;
	m_state = state;
	return S_OK;
}

STDMETHODIMP CNetMonBtn::GetDescription(char* szDescription)
{
	if(szDescription)
	{
		strcpy(szDescription, m_Name);
		return S_OK;
	}
	return S_FALSE;
}

STDMETHODIMP CNetMonBtn::OnLButtonDown(long key, int x, int y)
{
	return S_FALSE;
}

STDMETHODIMP CNetMonBtn::OnLButtonUp(long key, int x, int y)
{
	char fileName[MAX_PATH * 2];
	char params[MAX_PATH * 2];
	fileName[0] = 0;
	params[0] = 0;

	strcpy(fileName, m_run);
	PathRemoveArgs(fileName);

	char* args = PathGetArgs(m_run);
	if(args)
	{
		strcpy(params, PathGetArgs(m_run));
	}

	SHELLEXECUTEINFO exec;
	ZeroMemory(&exec, sizeof(exec));
	exec.cbSize = sizeof(exec);

	exec.fMask  = SEE_MASK_CONNECTNETDRV | SEE_MASK_DOENVSUBST;
	exec.lpFile = fileName;
	if(params[0])
	{
		exec.lpParameters = params;
	}
	exec.nShow  = SW_SHOWNORMAL;

	ShellExecuteEx(&exec);

	//ShellExecute(NULL, NULL, "control", "netconnections", NULL, SW_SHOWNORMAL);
	return S_OK;
}

STDMETHODIMP CNetMonBtn::Run()
{
	return S_OK;
}



STDMETHODIMP CNetMonBtn::DoAction(int action)
{
	if(action == CTMS_PROPERTIES)
	{
		if(m_container)
		{
			HWND hWndBand = m_container->GetBandWND();
			if(!m_propDlg)
			{
				m_propDlg =  new CPropertiesPP;
				m_propDlg->m_props = m_properties;
				strcpy(m_propDlg->m_skinPath, m_skinPath);
				strcpy(m_propDlg->m_onConnectSound, m_onConnectSound);
				strcpy(m_propDlg->m_onDisconnectSound, m_onDisconnectSound);
				strcpy(m_propDlg->m_run, m_run);
				memcpy(m_propDlg->m_params, m_params, sizeof(m_params));
				m_propDlg->m_hResource = m_hResource;
				m_propDlg->m_customConnections = m_customConnections;
				if(m_propDlg->Open(hWndBand, m_hResource) == IDOK)
				{
					m_customConnections = m_propDlg->m_customConnections;
					memcpy(m_params, m_propDlg->m_params, sizeof(m_params));
					strcpy(m_run, m_propDlg->m_run);
					strcpy(m_skinPath, m_propDlg->m_skinPath);
					strcpy(m_onConnectSound, m_propDlg->m_onConnectSound);
					strcpy(m_onDisconnectSound, m_propDlg->m_onDisconnectSound);
					m_properties = m_propDlg->m_props;
					m_skin.Load(m_skinPath, m_params);
					m_container->UpdateButtonInfo(UPDATE_SIZE | UPDATE_REDRAW);
					m_container->SaveButton();
				}
				delete m_propDlg;
				m_propDlg = NULL;
			} else
			{
				m_propDlg->Open(NULL, NULL);
			}
		}

	}
	return S_OK;
}

STDMETHODIMP_(int) CNetMonBtn::QueryContextMenu(HMENU hMenu, int index, int cmdFirst, int cmdLast)
{
/*	HWND hWndWinamp = GetWinAmpWnd(FALSE);
	int idx = 0;
	if(hWndWinamp)
	{
		char str[255];
		
		LoadString(m_hResource, IDS_CLOSEWA, str, 255);
		InsertMenu( hMenu, 
				   index + idx++,
				   MF_STRING | MF_BYPOSITION, 
				   cmdFirst + MCMD_CLOSEWA, 
				   str);
		//Get Repeat Status
		int ret = SendMessage(hWndWinamp, WM_USER, 0, 251);
		LoadString(m_hResource, IDS_REPEAT, str, 255);
		InsertMenu( hMenu, 
				   index + idx++,
				   MF_STRING | MF_BYPOSITION | (ret ? MF_CHECKED : 0), 
				   cmdFirst + MCMD_REPEAT, 
				   str);
		//Get Shuffle Status
		ret = SendMessage(hWndWinamp, WM_USER, 0, 250);
		LoadString(m_hResource, IDS_SHUFFLE, str, 255);
		InsertMenu( hMenu, 
				   index + idx++,
				   MF_STRING | MF_BYPOSITION | (ret ? MF_CHECKED : 0), 
				   cmdFirst + MCMD_SHUFFLE, 
				   str);
	}

	return idx;*/
	return 0;
}

STDMETHODIMP CNetMonBtn::InvokeCommand(int ID)
{
/*	HWND hWndWinamp = GetWinAmpWnd(FALSE);
	if(!hWndWinamp) return S_OK;
	switch(ID)
	{
	case MCMD_REPEAT:
		{
			int ret = SendMessage(hWndWinamp, WM_USER, 0, 251);
			if(ret)
			{
				SendMessage(hWndWinamp, WM_USER, 0, 253);
			} else
			{
				SendMessage(hWndWinamp, WM_USER, 1, 253);
			}
		}
		break;
	case MCMD_SHUFFLE:
		{
			int ret = SendMessage(hWndWinamp, WM_USER, 0, 250);
			if(ret)
			{
				SendMessage(hWndWinamp, WM_USER, 0, 252);
			} else
			{
				SendMessage(hWndWinamp, WM_USER, 1, 252);
			}
		}
		break;
	case MCMD_CLOSEWA:
		{
			SendMessage(hWndWinamp, WM_COMMAND, 40001, 0);
		}
		break;
	}*/
	return S_OK;
}


STDMETHODIMP CNetMonBtn::SetLCID(DWORD dwLCID, HMODULE hResModule)
{
	if(!hResModule)
	{
		m_hResource = g_hInst;
	} else
	{
		m_hResource = hResModule;
	}
	return S_OK;
}

STDMETHODIMP CNetMonBtn::GetIconLocation(LPSTR szLocation)
{
	return S_FALSE;
}

STDMETHODIMP CNetMonBtn::SetIconLocation(LPCSTR szLocation)
{
	return S_FALSE;
}

STDMETHODIMP CNetMonBtn::SetName(LPCSTR szName)
{
	strcpy(m_Name, szName);
	return S_OK;
}


DWORD WINAPI CNetMonBtn::TimerTrdProc(LPVOID lpParam)
{
	CoInitialize(NULL);
	CNetMonBtn* tr = (CNetMonBtn*) lpParam;
	SetThreadPriority(GetCurrentThread(), THREAD_PRIORITY_BELOW_NORMAL);
	tr->TimerThread();
	CoUninitialize();
	return 0;
}

void CNetMonBtn::TimerThread()
{
	while(true)
	{
		DWORD res = WaitForSingleObject(hStop, 1000);
		if(res == WAIT_OBJECT_0)
		{
			break;
		}
		DWORD update = LoadConnections();
		if(m_container && m_ID)
		{
			m_container->UpdateButtonInfo(update);
		}
	}
}

void CNetMonBtn::StartTimer()
{
	DWORD id;	
	hTimerThread = CreateThread(NULL, 0, CNetMonBtn::TimerTrdProc, (LPVOID) this, 0, &id);
}


STDMETHODIMP CNetMonBtn::GetButtonInfo(BUTTONINFO* btninfo)
{
	if(btninfo->dwMask & BTNINFO_MASK_MODEFLAG)
	{
		btninfo->dwModeFlags = BTN_FLAG_PLANE;
	}
	if(btninfo->dwMask & BTNINFO_MASK_ACTIONS)
	{
		btninfo->dwActions = CTMS_PROPERTIES;
	}
	if(btninfo->dwMask & BTNINFO_MASK_SIZE ||
	   btninfo->dwMask & BTNINFO_MASK_ACTUALSIZE)
	{
		RecalcLayout(btninfo->szBtnSize.cy);
		if(m_layout.itemWidth * m_layout.cols)
		{
			btninfo->szBtnSize.cx = m_layout.itemWidth * m_layout.cols;
			btninfo->szBtnSize.cy = m_layout.itemHeight * m_layout.rows;
		}
	}
	if(btninfo->dwMask & BTNINFO_MASK_DESCRIPTION)
	{
		char* szDescription = (char*) btninfo->strData;
		strcpy(szDescription, m_Name);
	}
	return S_OK;
}

STDMETHODIMP CNetMonBtn::Invoke(DWORD id, WPARAM wParam, LPARAM lParam)
{
	switch(id)
	{
	case TBTN_INVOKE_GETTIPSCAPTION:
		if(wParam >= 0 && (int) wParam < m_connections.GetCount())
		{
			LPWSTR* caption = (LPWSTR*) lParam;
			*caption = (LPWSTR) CoTaskMemAlloc(sizeof(WCHAR) * (strlen(m_connections[wParam].name) + 1));
			MultiByteToWideChar(CP_ACP, 0, m_connections[wParam].name, -1, *caption, strlen(m_connections[wParam].name) + 1);
		}
		return S_OK;
	case TBTN_INVOKE_GETTIPSICON:
		if(wParam >= 0 && (int) wParam < m_connections.GetCount())
		{
			HICON* ic = (HICON*) lParam;
			switch(m_connections[wParam].connectionInfo.dwType)
			{
			case MIB_IF_TYPE_TOKENRING:
			case MIB_IF_TYPE_ETHERNET:
			case MIB_IF_TYPE_FDDI:
				*ic = m_icNetcard;
				break;
			case MIB_IF_TYPE_SLIP:
			case MIB_IF_TYPE_PPP:
				*ic = m_icModem;
				break;
			}
		}
		return S_OK;
	case TBTN_INVOKE_GETTOOLTIPSCOUNT:
		{
			int* cnt = (int*) lParam;
			*cnt = m_connections.GetCount();
		}
		return S_OK;
	case TBTN_INVOKE_GETTOOLTIPRECT:
		{
			if(m_connections.GetCount())
			{
				int col = wParam / m_layout.rows;
				int row = wParam - col * m_layout.rows;
				LPRECT rc = (LPRECT) lParam;
				rc->left = m_rc.left + col * m_layout.itemWidth + m_layout.x;
				rc->top = m_rc.top + row * m_layout.itemHeight + m_layout.y;
				rc->right = rc->left + m_layout.itemWidth;
				rc->bottom = rc->top + m_layout.itemHeight;
			}
		}
		return S_OK;
	case TBTN_INVOKE_GETTOOLTIPTEXT:
		if(wParam >= 0 && (int) wParam < m_connections.GetCount())
		{
			char type[255];
			switch(m_connections[wParam].connectionInfo.dwType)
			{
			case MIB_IF_TYPE_ETHERNET:
				strcpy(type, "ETHERNET");
				break;
			case MIB_IF_TYPE_TOKENRING:
				strcpy(type, "TOKENRING");
				break;
			case MIB_IF_TYPE_FDDI:
				strcpy(type, "FDDI");
				break;
			case MIB_IF_TYPE_PPP:
				strcpy(type, "PPP");
				break;
			case MIB_IF_TYPE_LOOPBACK:
				strcpy(type, "LOOPBACK");
				break;
			case MIB_IF_TYPE_SLIP:
				strcpy(type, "SLIP");
				break;
			default:
				strcpy(type, "UNKNOWN");
				break;
			}

			char status[255] = "UNKNOWN";
			switch(m_connections[wParam].connectionInfo.dwOperStatus)
			{
			case MIB_IF_OPER_STATUS_NON_OPERATIONAL:
				strcpy(status, "Non operational");
				break;
			case MIB_IF_OPER_STATUS_UNREACHABLE:
				strcpy(status, "Not connected");
				break;
			case MIB_IF_OPER_STATUS_DISCONNECTED:
				strcpy(status, "Network cable is unplugged");
				break;
			case MIB_IF_OPER_STATUS_CONNECTING:
				strcpy(status, "Connecting");
				break;
			case MIB_IF_OPER_STATUS_CONNECTED:
				strcpy(status, "Connected to a remote peer");
				break;
			case MIB_IF_OPER_STATUS_OPERATIONAL:
				strcpy(status, "Operational");
				break;
			}

			char PhisAddress[255];
			PhisAddress[0] = 0;
			for(UINT i=0; i < m_connections[wParam].connectionInfo.dwPhysAddrLen; i++)
			{
				char str[20];
				sprintf(str, "%02X", m_connections[wParam].connectionInfo.bPhysAddr[i]);
				strcat(PhisAddress, str);
				if(i + 1 != m_connections[wParam].connectionInfo.dwPhysAddrLen)
				{
					strcat(PhisAddress, "-");
				}
			}

			char* str = (char*) lParam;

			strcpy(str, m_connections[wParam].name);

			strcat(str, "\nNetwork Type:\t");
			strcat(str, type);

			strcat(str, "\nStatus:\t\t");
			strcat(str, status);

			strcat(str, "\nIP address:\t");
			strcat(str, m_connections[wParam].ipAddress);

			strcat(str, "\nMAC address:\t");
			strcat(str, PhisAddress);

			char str1[255];
			SLOT slot;
			slot.format = FORMAT_AUTO;

			slot.field = FIELDS_SPEED;
			slot.GetFieldString(str1, m_connections[wParam].GetFieldValue(slot.field), FALSE);
			strcat(str, "\nLink Speed:\t");
			strcat(str, str1);

			slot.field = FIELDS_RECEIVED;
			slot.GetFieldString(str1, m_connections[wParam].GetFieldValue(slot.field), FALSE);
			strcat(str, "\nReceived:\t");
			strcat(str, str1);

			slot.field = FIELDS_SEND;
			slot.GetFieldString(str1, m_connections[wParam].GetFieldValue(slot.field), FALSE);
			strcat(str, "\nSent:\t\t");
			strcat(str, str1);

			slot.field = 0;
			slot.GetFieldString(str1, m_connections[wParam].connectionInfo.dwMtu, FALSE);
			strcat(str, "\nMTU:\t\t");
			strcat(str, str1);

			slot.field = 0;
			slot.GetFieldString(str1, m_connections[wParam].connectionInfo.dwInErrors, FALSE);
			strcat(str, "\nIn Errors:\t");
			strcat(str, str1);

			slot.field = 0;
			slot.GetFieldString(str1, m_connections[wParam].connectionInfo.dwOutErrors, FALSE);
			strcat(str, "\nOut Errors:\t");
			strcat(str, str1);
		}
		return S_OK;
	case TBTN_INVOKE_GETTIPSTEXT:
		if(wParam >= 0 && (int) wParam < m_connections.GetCount())
		{
			char type[255];
			switch(m_connections[wParam].connectionInfo.dwType)
			{
			case MIB_IF_TYPE_ETHERNET:
				strcpy(type, "ETHERNET");
				break;
			case MIB_IF_TYPE_TOKENRING:
				strcpy(type, "TOKENRING");
				break;
			case MIB_IF_TYPE_FDDI:
				strcpy(type, "FDDI");
				break;
			case MIB_IF_TYPE_PPP:
				strcpy(type, "PPP");
				break;
			case MIB_IF_TYPE_LOOPBACK:
				strcpy(type, "LOOPBACK");
				break;
			case MIB_IF_TYPE_SLIP:
				strcpy(type, "SLIP");
				break;
			default:
				strcpy(type, "UNKNOWN");
				break;
			}

			char status[255] = "UNKNOWN";
			switch(m_connections[wParam].connectionInfo.dwOperStatus)
			{
			case MIB_IF_OPER_STATUS_NON_OPERATIONAL:
				strcpy(status, "Non operational");
				break;
			case MIB_IF_OPER_STATUS_UNREACHABLE:
				strcpy(status, "Not connected");
				break;
			case MIB_IF_OPER_STATUS_DISCONNECTED:
				strcpy(status, "Network cable is unplugged");
				break;
			case MIB_IF_OPER_STATUS_CONNECTING:
				strcpy(status, "Connecting");
				break;
			case MIB_IF_OPER_STATUS_CONNECTED:
				strcpy(status, "Connected to a remote peer");
				break;
			case MIB_IF_OPER_STATUS_OPERATIONAL:
				strcpy(status, "Operational");
				break;
			}

			char PhisAddress[255];
			PhisAddress[0] = 0;
			for(UINT i=0; i < m_connections[wParam].connectionInfo.dwPhysAddrLen; i++)
			{
				char str[20];
				sprintf(str, "%02X", m_connections[wParam].connectionInfo.bPhysAddr[i]);
				strcat(PhisAddress, str);
				if(i + 1 != m_connections[wParam].connectionInfo.dwPhysAddrLen)
				{
					strcat(PhisAddress, "-");
				}
			}

			char* str = new char[1024];
			str[0] = 0;

			strcat(str, "Network Type:\t<b>");
			strcat(str, type);
			strcat(str, "</b>");

			strcat(str, "\nStatus:\t<b>");
			strcat(str, status);
			strcat(str, "</b>");

			strcat(str, "\nIP address:\t<b>");
			strcat(str, m_connections[wParam].ipAddress);
			strcat(str, "</b>");

			strcat(str, "\nMAC address:\t<b>");
			strcat(str, PhisAddress);
			strcat(str, "</b>");

			char str1[255];
			SLOT slot;
			slot.format = FORMAT_AUTO;

			slot.field = FIELDS_SPEED;
			slot.GetFieldString(str1, m_connections[wParam].GetFieldValue(slot.field), FALSE);
			strcat(str, "\nLink Speed:\t<b>");
			strcat(str, str1);
			strcat(str, "</b>");

			slot.field = FIELDS_RECEIVED;
			slot.GetFieldString(str1, m_connections[wParam].GetFieldValue(slot.field), FALSE);
			strcat(str, "\nReceived:\t<b>");
			strcat(str, str1);
			strcat(str, "</b>");

			slot.field = FIELDS_SEND;
			slot.GetFieldString(str1, m_connections[wParam].GetFieldValue(slot.field), FALSE);
			strcat(str, "\nSent:\t<b>");
			strcat(str, str1);
			strcat(str, "</b>");

			slot.field = 0;
			slot.GetFieldString(str1, m_connections[wParam].connectionInfo.dwMtu, FALSE);
			strcat(str, "\nMTU:\t<b>");
			strcat(str, str1);
			strcat(str, "</b>");

			slot.field = 0;
			slot.GetFieldString(str1, m_connections[wParam].connectionInfo.dwInErrors, FALSE);
			strcat(str, "\nIn Errors:\t<b>");
			strcat(str, str1);
			strcat(str, "</b>");

			slot.field = 0;
			slot.GetFieldString(str1, m_connections[wParam].connectionInfo.dwOutErrors, FALSE);
			strcat(str, "\nOut Errors:\t<b>");
			strcat(str, str1);
			strcat(str, "</b>");

			LPWSTR* strW = (LPWSTR*) lParam;
			*strW = (LPWSTR) CoTaskMemAlloc(sizeof(WCHAR) * (strlen(str) + 1));
			MultiByteToWideChar(CP_ACP, 0, str, -1, *strW, strlen(str) + 1);
			delete str;
		}
		return S_OK;
	}
	return S_FALSE;
}


DWORD CNetMonBtn::LoadConnections(void)
{
	DWORD res = UPDATE_REDRAW;

	DWORD intNumber = 0;
	GetNumberOfInterfaces(&intNumber);
	ULONG szBuf = sizeof(IP_INTERFACE_INFO) * intNumber;
	PIP_INTERFACE_INFO intTable = (PIP_INTERFACE_INFO) malloc(szBuf);
	DWORD ret = GetInterfaceInfo(intTable, &szBuf);
	if(ret == ERROR_INSUFFICIENT_BUFFER)
	{
		intTable = (PIP_INTERFACE_INFO) realloc(intTable, szBuf);
		ret = GetInterfaceInfo(intTable, &szBuf);
	}
	DWORD tm = GetTickCount();
	BOOL added = FALSE;
	BOOL deleted = FALSE;
	if(ret == NO_ERROR)
	{
		for(int i=0; i < m_connections.GetCount(); i++)
		{
			m_connections[i].exists = FALSE;
		}

		LockItems();
		MIB_IFROW ifRow;
		for(int i=0; i < intTable->NumAdapters; i++)
		{
			ifRow.dwIndex = intTable->Adapter[i].Index;
			GetIfEntry(&ifRow);
			if(ifRow.dwType == MIB_IF_TYPE_LOOPBACK) continue;
			int idx = -1;
			BOOL isHidden = FALSE;
			int custIDX = -1;
			for(int k=0; k < m_customConnections.GetCount(); k++)
			{
				if(!wcscmp(intTable->Adapter[i].Name, m_customConnections[k].id))
				{
					custIDX = k;
					break;
				}
			}
			if(custIDX >= 0)
			{
				if(m_customConnections[k].flags & CC_FLAG_HIDE)
				{
					isHidden = TRUE;
				}
			}
			if(!isHidden)
			{
				for(int j=0; j < m_connections.GetCount(); j++)
				{
					if(m_connections[j].connectionInfo.dwIndex == ifRow.dwIndex)
					{
						idx = j;
						m_connections[j].exists = TRUE;
						break;
					}
				}
				if(idx < 0)
				{
					CONNECTION conn;
					ZeroMemory(&conn, sizeof(conn));
					memcpy(conn.idName, intTable->Adapter[i].Name, sizeof(conn.idName));
					if(!FindConnectionName(conn.idName, conn.name, 256))
					{
						int sz = min(ifRow.dwDescrLen, sizeof(conn.name));
						strncpy(conn.name, (LPSTR) ifRow.bDescr, sz);
						conn.name[sz] = 0;
					}
					conn.connectionInfo = ifRow;
					conn.updateTime = tm;
					conn.prevInfo = ifRow;
					conn.prevUpdateTime = tm;
					conn.exists = TRUE;
					if(custIDX >= 0)
					{
						conn.bwLimit = m_customConnections[custIDX].bandWidthLimit;
						conn.bwlType = m_customConnections[custIDX].flags & 0xF;
						if(!conn.bwLimit)
						{
							conn.bwLimit = 100;
						}
					}
					time(&conn.beginTime);
					m_connections.Add(&conn);
					idx = m_connections.GetCount() - 1;
					res = UPDATE_SIZE;
					added = TRUE;
				} else
				{
					if(custIDX >= 0)
					{
						if(m_connections[idx].bwlType != (m_customConnections[custIDX].flags & 0xF))
						{
							m_connections[idx].bwLimit = m_customConnections[custIDX].bandWidthLimit;
							m_connections[idx].bwlType = m_customConnections[custIDX].flags & 0xF;
							if(m_connections[idx].bwlType != CC_FLAG_BWCUSTOM)
							{
								m_connections[idx].bwLimit = 100;
							}
							if(!m_connections[idx].bwLimit)
							{
								m_connections[idx].bwLimit = 100;
							}
						} else if(m_connections[idx].bwlType == CC_FLAG_BWCUSTOM && m_connections[idx].bwLimit != m_customConnections[custIDX].bandWidthLimit)
						{
							m_connections[idx].bwLimit = m_customConnections[custIDX].bandWidthLimit;
							if(!m_connections[idx].bwLimit)
							{
								m_connections[idx].bwLimit = 100;
							}
						}
					}
					m_connections[idx].prevInfo = m_connections[idx].connectionInfo;
					m_connections[idx].prevUpdateTime = m_connections[idx].updateTime;
					m_connections[idx].connectionInfo = ifRow;
					m_connections[idx].updateTime = tm;
					
					if(m_connections[idx].bwlType == CC_FLAG_BWAUTO)
					{
						DWORD curBW = m_connections[idx].GetFieldValue(FIELDS_BANDWIDTH);
						if(m_connections[idx].bwLimit < curBW)
						{
							m_connections[idx].bwLimit = curBW;
						}
					}
					memmove(m_connections[idx].in + 1, m_connections[idx].in, MAX_HISTORY - 1);
					m_connections[idx].in[0] = m_connections[idx].GetFieldValue(FIELDS_INBANDWIDTH);
					
					memmove(m_connections[idx].out + 1, m_connections[idx].out, MAX_HISTORY - 1);
					m_connections[idx].out[0] = m_connections[idx].GetFieldValue(FIELDS_OUTBANDWIDTH);
				}
			}
		}
		while(true) 
		{
			BOOL noDelete = TRUE;
			for(int i=0; i < m_connections.GetCount(); i++)
			{
				if(!m_connections[i].exists)
				{
					deleted = TRUE;
					m_connections.Delete(i);
					noDelete = FALSE;
					res = UPDATE_SIZE;
					break;
				}
			}	
			if(noDelete) break;
		}

		UnlockItems();

		if(added)
		{
			PlaySound(m_onConnectSound, NULL, SND_FILENAME | SND_ASYNC);
		}
		if(deleted)
		{
			PlaySound(m_onDisconnectSound, NULL, SND_FILENAME | SND_ASYNC);
		}
	}
	free(intTable);

	ULONG cbSize = sizeof(IP_ADAPTER_INFO);
	PIP_ADAPTER_INFO adInf = (PIP_ADAPTER_INFO) malloc(cbSize);
	ret = GetAdaptersInfo(adInf, &cbSize);
	if(ret == ERROR_BUFFER_OVERFLOW)
	{
		free(adInf);
		adInf = (PIP_ADAPTER_INFO) malloc(cbSize);
		ret = GetAdaptersInfo(adInf, &cbSize);
	}
	if(!ret)
	{
		PIP_ADAPTER_INFO cur = adInf;
		while(cur)
		{
			for(int i=0; i < m_connections.GetCount(); i++)
			{
				if(m_connections[i].connectionInfo.dwIndex == cur->Index)
				{
					strcpy(m_connections[i].ipAddress, cur->IpAddressList.IpAddress.String);
				}
			}
			cur = cur->Next;
		}
	}
	free(adInf);

	return res;
	
	
//////////////////////////////////////////////////////////////////////////
	
/*	ULONG cbSize = sizeof(MIB_IFTABLE);
	PMIB_IFTABLE ifTable = (PMIB_IFTABLE) malloc(cbSize);
	ZeroMemory(ifTable, cbSize);
	DWORD ret = GetIfTable(ifTable, &cbSize, FALSE);
	if(ret == ERROR_INSUFFICIENT_BUFFER)
	{
		free(ifTable);
		ifTable = (PMIB_IFTABLE) malloc(cbSize);
		ZeroMemory(ifTable, cbSize);
		ret = GetIfTable(ifTable, &cbSize, FALSE);
	}
	DWORD tm = GetTickCount();
	BOOL added = FALSE;
	BOOL deleted = FALSE;
	if(ret == NO_ERROR)
	{
		for(int i=0; i < m_connections.GetCount(); i++)
		{
			m_connections[i].exists = FALSE;
		}

		LockItems();
		for(DWORD i=0; i < ifTable->dwNumEntries; i++)
		{
			if(ifTable->table[i].dwType == MIB_IF_TYPE_LOOPBACK) continue;
			int idx = -1;
			for(int j=0; j < m_connections.GetCount(); j++)
			{
				if(m_connections[j].connectionInfo.dwIndex == ifTable->table[i].dwIndex)
				{
					idx = j;
					m_connections[j].exists = TRUE;
					break;
				}
			}
			if(idx < 0)
			{
				CONNECTION conn;
				ZeroMemory(&conn, sizeof(conn));
				conn.connectionInfo = ifTable->table[i];
				conn.updateTime = tm;
				conn.prevInfo = ifTable->table[i];
				conn.prevUpdateTime = tm;
				conn.exists = TRUE;
				time(&conn.beginTime);
                m_connections.Add(&conn);
				idx = m_connections.GetCount() - 1;
				res = UPDATE_SIZE;
				added = TRUE;
			} else
			{
				m_connections[idx].prevInfo = m_connections[idx].connectionInfo;
				m_connections[idx].prevUpdateTime = m_connections[idx].updateTime;
				m_connections[idx].connectionInfo = ifTable->table[i];
				m_connections[idx].updateTime = tm;
			}
		}
		while(true) 
		{
			BOOL noDelete = TRUE;
			for(int i=0; i < m_connections.GetCount(); i++)
			{
				if(!m_connections[i].exists)
				{
					deleted = TRUE;
					m_connections.Delete(i);
					noDelete = FALSE;
					res = UPDATE_SIZE;
					break;
				}
			}	
			if(noDelete) break;
		}

		UnlockItems();

		if(added)
		{
			PlaySound(m_onConnectSound, NULL, SND_FILENAME | SND_ASYNC);
		}
		if(deleted)
		{
			PlaySound(m_onDisconnectSound, NULL, SND_FILENAME | SND_ASYNC);
		}
	}
	free(ifTable);

	cbSize = sizeof(IP_ADAPTER_INFO);
	PIP_ADAPTER_INFO adInf = (PIP_ADAPTER_INFO) malloc(cbSize);
	ret = GetAdaptersInfo(adInf, &cbSize);
	if(ret == ERROR_BUFFER_OVERFLOW)
	{
		free(adInf);
		adInf = (PIP_ADAPTER_INFO) malloc(cbSize);
		ret = GetAdaptersInfo(adInf, &cbSize);
	}
	if(!ret)
	{
		PIP_ADAPTER_INFO cur = adInf;
		while(cur)
		{
			for(int i=0; i < m_connections.GetCount(); i++)
			{
				if(m_connections[i].connectionInfo.dwIndex == cur->Index)
				{
					strcpy(m_connections[i].ipAddress, cur->IpAddressList.IpAddress.String);
				}
			}
			cur = cur->Next;
		}
	}
	free(adInf);

	return res;*/
}

DWORD CONNECTION::GetFieldValue(DWORD fieldID)
{
	switch(fieldID)
	{
	case FIELDS_BWLIMIT:
		if(bwlType == CC_FLAG_BWAUTO || bwlType == CC_FLAG_BWCUSTOM)
		{
			return bwLimit;
		} else
		return connectionInfo.dwSpeed / 8;
	case FIELDS_SPEED:
		return connectionInfo.dwSpeed;
	case FIELDS_TIME:
		{
			time_t t;
			time(&t);
			return (DWORD) (t - beginTime);
		}
	case FIELDS_SEND:
		return connectionInfo.dwOutOctets;
	case FIELDS_RECEIVED:
		return connectionInfo.dwInOctets;
	case FIELDS_INBANDWIDTH:
		if(updateTime - prevUpdateTime)
		{
			return (DWORD) ((double) (connectionInfo.dwInOctets - prevInfo.dwInOctets) / ((double) (updateTime - prevUpdateTime) / 1000.0));
		} else
		{
			return 0;
		}
	case FIELDS_OUTBANDWIDTH:
		if(updateTime - prevUpdateTime)
		{
			return (DWORD) ((double) (connectionInfo.dwOutOctets - prevInfo.dwOutOctets) / ((double) (updateTime - prevUpdateTime) / 1000.0));
		} else
		{
			return 0;
		}
	case FIELDS_BANDWIDTH:
		return GetFieldValue(FIELDS_INBANDWIDTH) + GetFieldValue(FIELDS_OUTBANDWIDTH);
	case FIELDS_INLOAD:
		if(GetFieldValue(FIELDS_BWLIMIT))
		{
			DWORD ret = GetFieldValue(FIELDS_INBANDWIDTH) * 100 / GetFieldValue(FIELDS_BWLIMIT);
			if(ret > 100) ret = 100;
			return ret;
		} else
		{
			return 0;
		}
	case FIELDS_OUTLOAD:
		if(GetFieldValue(FIELDS_BWLIMIT))
		{
			DWORD ret = GetFieldValue(FIELDS_OUTBANDWIDTH) * 100 / GetFieldValue(FIELDS_BWLIMIT);
			if(ret > 100) ret = 100;
			return ret;
		} else
		{
			return 0;
		}
	case FIELDS_LOAD:
		if(GetFieldValue(FIELDS_BWLIMIT))
		{
			DWORD ret = GetFieldValue(FIELDS_BANDWIDTH) * 100 / GetFieldValue(FIELDS_BWLIMIT);
			if(ret > 100) ret = 100;
			return ret;
		} else
		{
			return 0;
		}
	case FIELDS_SOMESENT:
		if(connectionInfo.dwOutOctets != prevInfo.dwOutOctets)
		{
			return 1;
		}
		return 0;
	case FIELDS_SOMERECEIVED:
		if(connectionInfo.dwInOctets != prevInfo.dwInOctets)
		{
			return 1;
		}
		return 0;
	case FIELDS_IP:
		return (DWORD) ipAddress;
	case FIELDS_NAME:
		return (DWORD) name;
	}
	return 0;
}

void CNetMonBtn::RecalcLayout(int height)
{
	m_layout.rows = m_properties.rows;
	if(m_layout.rows > m_connections.GetCount()) m_layout.rows = m_connections.GetCount();
	if(!m_layout.rows) m_layout.rows = 1;
	m_layout.cols = m_connections.GetCount() / m_layout.rows;
	if(m_layout.cols * m_layout.rows < m_connections.GetCount()) m_layout.cols++;

	m_layout.itemHeight = height / m_layout.rows;
	m_layout.itemWidth = 0;
	m_skin.RecalcLayout(m_layout.itemWidth, m_layout.itemHeight, &m_properties);
}

void SLOT::GetFieldString(char* strValue, DWORD value, BOOL maxString)
{
	strcpy(strValue, "999999");
	if(maxString)
	{
		switch(field)
		{
		case FIELDS_SPEED:
			strcpy(strValue, "999.99Mbps");
			break;
		case FIELDS_RECEIVED:
		case FIELDS_SEND:
			switch(format)
			{
			case FORMAT_AUTO:
				strcpy(strValue, "999.99 G");
				break;
			case FORMAT_BYTE:
				strcpy(strValue, "9999999999");
				break;
			case FORMAT_KB:
				strcpy(strValue, "999999 K");
				break;
			case FORMAT_MB:
				strcpy(strValue, "9999.99 M");
				break;
			}
			break;
		case FIELDS_TIME:
			strcpy(strValue, "999:99:99");
			break;
		case FIELDS_INBANDWIDTH:
		case FIELDS_OUTBANDWIDTH:
		case FIELDS_BANDWIDTH:
			strcpy(strValue, "9999 Mb/s");
			break;
		case FIELDS_INLOAD:
		case FIELDS_OUTLOAD:
		case FIELDS_LOAD:
			strcpy(strValue, "100%");
			break;
		case FIELDS_IP:
			strcpy(strValue, "999.999.999.999");
			break;
		case FIELDS_NAME:
			strcpy(strValue, "Local network");
			break;
		}
	} else
	{
		switch(field)
		{
		case FIELDS_SPEED:
			{
				double speed = value;
				char* div[] = {"", "Kbps", "Mbps", "Gbps", NULL};
				int i=0;
				while(div[i + 1] && speed >= 1000)
				{
					speed /= 1000;
					i++;
				}
				DWORD spInt = (DWORD) speed;
				if(((double) spInt) != speed)
				{
					sprintf(strValue, "%0.2f %s", speed, div[i]);
				} else
				{
					sprintf(strValue, "%.0f %s", speed, div[i]);
				}
			}
			break;
		case FIELDS_RECEIVED:
		case FIELDS_SEND:
			{
				char* div[5] = {"", " K", " M", " G", NULL};
				switch(format)
				{
				case FORMAT_BYTE:
					div[0] = NULL;
					break;
				case FORMAT_KB:
					div[2] = NULL;
					break;
				case FORMAT_MB:
					div[3] = NULL;
					break;
				}
				double val = value;
				int i=0;
				while(div[i + 1] && val >= 1024)
				{
					val /= 1024;
					i++;
				}
				if(i >= 2)
				{
					sprintf(strValue, "%0.2f%s", val, div[i]);
				} else
				{
					sprintf(strValue, "%.0f%s", val, div[i]);
				}
			}
			break;
		case FIELDS_TIME:
			{
				DWORD h = value / 3600;
				DWORD m = (value - (h * 3600)) / 60;
				DWORD s = value - (h * 3600) - m * 60;
				if(h)
				{
					sprintf(strValue, "%d:%02d:%02d", h, m, s);
				} else
				{
					sprintf(strValue, "%02d:%02d", m, s);
				}
			}
			break;
		case FIELDS_INBANDWIDTH:
		case FIELDS_OUTBANDWIDTH:
		case FIELDS_BANDWIDTH:
			{
				char* div[5] = {"b/s", "Kb/s", "Mb/s", "Gb/s", NULL};
				double val = value;
				int i=0;
				while(div[i + 1] && val >= 1024 * 10)
				{
					val /= 1024;
					i++;
				}
				sprintf(strValue, "%.0f %s", val, div[i]);
			}
			break;
		case FIELDS_INLOAD:
		case FIELDS_OUTLOAD:
		case FIELDS_LOAD:
			sprintf(strValue, "%d%s", value, "%");
			break;
		case FIELDS_IP:
			{
				char* ip = (char*) value;
				strcpy(strValue, ip);
			}
			break;
		case FIELDS_NAME:
			{
				char* name = (char*) value;
				strcpy(strValue, name);
			}
			break;
		default:
			sprintf(strValue, "%d", value);
			break;
		}
	}
}

CSkin::CSkin()
{
	for(int i=0; i < MAX_DIAGRAMS; i++)
	{
		diagrams[i].type = DIAGRAM_TYPE_HIDE;
	}

	for(int i=0; i < MAX_SLOTS; i++)
	{
		slots[i].hFont = NULL;
		slots[i].field = FIELDS_UNUSED;
		slots[i].format = FORMAT_AUTO;
	}
}

CSkin::~CSkin()
{
}

void CSkin::Load(LPSTR skinFile, PARAMS* params)
{
	Clear();
	char fileName[MAX_PATH];

	GetModuleFileName(g_hInst, fileName, MAX_PATH);
	PathRemoveFileSpec(fileName);
	PathAddBackslash(fileName);
	strcat(fileName, "skins\\");
	strcat(fileName, skinFile);

	if(!skinFile[0])
	{
		LoadDefault(params);
		return;
	}

	skinType = SKIN_BACKGOUND;

	char skinPath[MAX_PATH];
	strcpy(skinPath, fileName);
	PathRemoveFileSpec(skinPath);
	PathAddBackslash(skinPath);

	char flName[MAX_PATH];

	// loading background
	char str[MAX_PATH];
	GetPrivateProfileString("general", "background", "", str, MAX_PATH, fileName);
	if(!str[0])
	{
		LoadDefault(params);
		return;
	}
	strcpy(flName, skinPath);
	strcat(flName, str);
	imgBackground.Load(flName, CXIMAGE_FORMAT_PNG);

	for(int i=0; i < MAX_DIAGRAMS; i++)
	{
		char strSec[50];
		sprintf(strSec, "diagram%d", i+1);
		GetPrivateProfileString(strSec, "type", "", str, MAX_PATH, fileName);
		if(!str[0]) break;
		if(!strcmpi(str, "Light"))
		{
			diagrams[i].type = DIAGRAM_TYPE_LIGHT;
		} else if(!strcmpi(str, "VertRow"))
		{
			diagrams[i].type = DIAGRAM_TYPE_VERTICALROW;
		} else if(!strcmpi(str, "HorzRow"))
		{
			diagrams[i].type = DIAGRAM_TYPE_HORIZONTALROW;
		} else if(!strcmpi(str, "Percent"))
		{
			diagrams[i].type = DIAGRAM_TYPE_PERCENT;
		} else if(!strcmpi(str, "History"))
		{
			diagrams[i].type = DIAGRAM_TYPE_HISTORY;
		}
		GetPrivateProfileString(strSec, "picture", "", str, MAX_PATH, fileName);
		strcpy(flName, skinPath);
		strcat(flName, str);
		diagrams[i].imgDiagram.Load(flName, CXIMAGE_FORMAT_PNG);

		GetPrivateProfileString(strSec, "field", "INBOUND", str, MAX_PATH, fileName);
		if(!strcmpi(str, "INLOAD"))
		{
			diagrams[i].field = FIELDS_INLOAD;
		} else if(!strcmpi(str, "OUTLOAD"))
		{
			diagrams[i].field = FIELDS_OUTLOAD;
		} else if(!strcmpi(str, "LOAD"))
		{
			diagrams[i].field = FIELDS_LOAD;
		} else if(!strcmpi(str, "SOMESENT"))
		{
			diagrams[i].field = FIELDS_SOMESENT;
		} else if(!strcmpi(str, "SOMERECEIVED"))
		{
			diagrams[i].field = FIELDS_SOMERECEIVED;
		} else if(!strcmpi(str, "INSPEED"))
		{
			diagrams[i].field = FIELDS_INBANDWIDTH;
		} else if(!strcmpi(str, "OUTSPEED"))
		{
			diagrams[i].field = FIELDS_OUTBANDWIDTH;
		}
	
		GetPrivateProfileString(strSec, "pos", "0 0", str, MAX_PATH, fileName);
		ParsePoint(str, diagrams[i].x, diagrams[i].y);

		GetPrivateProfileString(strSec, "color", "0 0 0 0", str, MAX_PATH, fileName);
		diagrams[i].color = ParseColor(str);

		GetPrivateProfileString(strSec, "size", "0 0", str, MAX_PATH, fileName);
		ParsePoint(str, diagrams[i].width, diagrams[i].height);

		GetPrivateProfileString(strSec, "fillType", "solid", str, MAX_PATH, fileName);
		if(!StrCmpI(str, "solid"))
		{
			diagrams[i].fillType = HD_FILLTYPE_SOLID;
		} else if(!StrCmpI(str, "line"))
		{
			diagrams[i].fillType = HD_FILLTYPE_LINE;
		} else if(!StrCmpI(str, "picture"))
		{
			diagrams[i].fillType = HD_FILLTYPE_PICTURE;
		}

		GetPrivateProfileString(strSec, "beginFrom", "right", str, MAX_PATH, fileName);
		if(!StrCmpI(str, "right"))
		{
			diagrams[i].startFrom = START_FROM_RIGHT;
		} else if(!StrCmpI(str, "left"))
		{
			diagrams[i].startFrom = START_FROM_LEFT;
		} else if(!StrCmpI(str, "top"))
		{
			diagrams[i].startFrom = START_FROM_TOP;
		} else if(!StrCmpI(str, "bottom"))
		{
			diagrams[i].startFrom = START_FROM_BOTTOM;
		}

		GetPrivateProfileString(strSec, "valuesFrom", "bottom", str, MAX_PATH, fileName);
		if(!StrCmpI(str, "right"))
		{
			diagrams[i].startFrom |= START_FROM_RIGHT << 4;
		} else if(!StrCmpI(str, "left"))
		{
			diagrams[i].startFrom |= START_FROM_LEFT << 4;
		} else if(!StrCmpI(str, "top"))
		{
			diagrams[i].startFrom |= START_FROM_TOP << 4;
		} else if(!StrCmpI(str, "bottom"))
		{
			diagrams[i].startFrom |= START_FROM_BOTTOM << 4;
		}

		diagrams[i].margins.top = GetPrivateProfileInt(strSec, "TopSizeMargin", 0, fileName);
		diagrams[i].margins.bottom = GetPrivateProfileInt(strSec, "BottomSizeMargin", 0, fileName);
		diagrams[i].margins.left = GetPrivateProfileInt(strSec, "LeftSizeMargin", 0, fileName);
		diagrams[i].margins.right = GetPrivateProfileInt(strSec, "RightSizeMargin", 0, fileName);

		diagrams[i].tile = GetPrivateProfileInt(strSec, "tile", 0, fileName);
	}

	int curUserSlot = 0;

	for(int i=0; i < MAX_SLOTS; i++)
	{
		char strSec[50];
		sprintf(strSec, "DataSlot%d", i+1);

		GetPrivateProfileString(strSec, "field", "", str, MAX_PATH, fileName);
		if(!str[0]) break;
		if(!strcmpi(str, "BANDWIDTH"))
		{
			slots[i].field = FIELDS_SPEED;
		} else if(!strcmpi(str, "RECEIVED"))
		{
			slots[i].field = FIELDS_RECEIVED;
		} else if(!strcmpi(str, "SEND"))
		{
			slots[i].field = FIELDS_SEND;
		} else if(!strcmpi(str, "TIME"))
		{
			slots[i].field = FIELDS_TIME;
		} else if(!strcmpi(str, "INSPEED"))
		{
			slots[i].field = FIELDS_INBANDWIDTH;
		} else if(!strcmpi(str, "OUTSPEED"))
		{
			slots[i].field = FIELDS_OUTBANDWIDTH;
		} else if(!strcmpi(str, "SPEED"))
		{
			slots[i].field = FIELDS_BANDWIDTH;
		} else if(!strcmpi(str, "INLOAD"))
		{
			slots[i].field = FIELDS_INLOAD;
		} else if(!strcmpi(str, "OUTLOAD"))
		{
			slots[i].field = FIELDS_OUTLOAD;
		} else if(!strcmpi(str, "LOAD"))
		{
			slots[i].field = FIELDS_LOAD;
		} else if(!strcmpi(str, "IP"))
		{
			slots[i].field = FIELDS_IP;
		} else if(!strcmpi(str, "USERCHOICE"))
		{
			for(int j = curUserSlot; j < MAX_FIELDS; j++)
			{
				if(params[j].show)
				{
					slots[i].field = params[j].paramID;
					curUserSlot = j + 1;
					break;
				}
			}
		}

		GetPrivateProfileString(strSec, "font", "Verdana", slots[i].fontName, 255, fileName);

		slots[i].bold =GetPrivateProfileInt(strSec, "bold", 0, fileName);
		slots[i].italic =GetPrivateProfileInt(strSec, "italic", 0, fileName);

		GetPrivateProfileString(strSec, "align", "right", str, MAX_PATH, fileName);
		if(!strcmpi(str, "right"))
		{
			slots[i].align = ALIGN_RIGHT;
		} else if(!strcmpi(str, "left"))
		{
			slots[i].align = ALIGN_LEFT;
		} else if(!strcmpi(str, "center"))
		{
			slots[i].align = ALIGN_CENTER;
		}

		GetPrivateProfileString(strSec, "pos", "0 0", str, MAX_PATH, fileName);
		ParsePoint(str, slots[i].x, slots[i].y);

		GetPrivateProfileString(strSec, "size", "0 0", str, MAX_PATH, fileName);
		ParsePoint(str, slots[i].width, slots[i].height);

		GetPrivateProfileString(strSec, "color", "", str, MAX_PATH, fileName);
		if(str[0])
		{
			slots[i].color = ParseColor(str);
		} else
		{
			slots[i].color = 0xFFFFFFFF;
		}
	}
}

void CSkin::Draw(HDC dc, int x, int y, CONNECTION* conn)
{
	if(skinType & SKIN_BACKGOUND)
	{
		imgBackground.Draw(dc, x, y);
	}
	for(int i=0; i < MAX_SLOTS; i++)
	{
		if(slots[i].field != FIELDS_UNUSED)
		{
			char	strVal[255];
			DWORD	val = conn->GetFieldValue(slots[i].field);
			slots[i].GetFieldString(strVal, val, FALSE);
			RECT rcText;
			rcText.left = x + slots[i].x;
			rcText.top = y + slots[i].y;
			rcText.right = rcText.left + slots[i].width;
			rcText.bottom = rcText.top + slots[i].height;
			
			HFONT oldFont = (HFONT) SelectObject(dc, slots[i].hFont);
			COLORREF oldTextColor = 0;
			if(slots[i].color != 0xFFFFFFFF)
			{
				oldTextColor = SetTextColor(dc, slots[i].color);
			}
			DWORD align = DT_RIGHT;
			switch(slots[i].align)
			{
			case ALIGN_RIGHT:
				align = DT_RIGHT;
				break;
			case ALIGN_LEFT:
				align = DT_LEFT;
				break;
			case ALIGN_CENTER:
				align = DT_CENTER;
				break;
			}
			DrawText(dc, strVal, -1, &rcText, align | DT_SINGLELINE | DT_VCENTER);
			if(slots[i].color != 0xFFFFFFFF)
			{
				SetTextColor(dc, oldTextColor);
			}
			SelectObject(dc, oldFont);}
	}

	for(int i=0; i < MAX_DIAGRAMS; i++)
	{
		if(diagrams[i].type != DIAGRAM_TYPE_HISTORY)
		{
			DrawDiagram(dc, x, y, &diagrams[i], conn);
		} else
		{
			switch(diagrams[i].field)
			{
			case FIELDS_INBANDWIDTH:
				DrawDiagram(dc, x, y, &diagrams[i], conn);
				break;
			case FIELDS_OUTBANDWIDTH:
				DrawDiagram(dc, x, y, &diagrams[i], conn);
				break;
			}
		}
	}
}

void CSkin::LoadDefault(PARAMS* params)
{
	Clear();

	skinType = SKIN_AUTOSLOTS;

	diagrams[0].type = DIAGRAM_TYPE_VERTICALROW;
	diagrams[0].imgDiagram.LoadResource(FindResource(g_hInst, MAKEINTRESOURCE(IDR_INBOUND), "PNG"), CXIMAGE_FORMAT_PNG, g_hInst);
	diagrams[0].margins.left = 0;
	diagrams[0].margins.right = 0;
	diagrams[0].margins.top = 3;
	diagrams[0].margins.bottom = 3;
	diagrams[0].tile = FALSE;
	diagrams[0].field = FIELDS_INLOAD;
	diagrams[0].width = diagrams[0].imgDiagram.GetWidth();

	diagrams[1].type = DIAGRAM_TYPE_VERTICALROW;
	diagrams[1].imgDiagram.LoadResource(FindResource(g_hInst, MAKEINTRESOURCE(IDR_OUTBOUND), "PNG"), CXIMAGE_FORMAT_PNG, g_hInst);
	diagrams[1].margins.left = 0;
	diagrams[1].margins.right = 0;
	diagrams[1].margins.top = 3;
	diagrams[1].margins.bottom = 3;
	diagrams[1].tile = FALSE;
	diagrams[1].field = FIELDS_OUTLOAD;
	diagrams[1].width = diagrams[1].imgDiagram.GetWidth();

	int curParam = 0;
	for(int i=0; i < MAX_SLOTS; i++)
	{
		slots[i].field = FIELDS_UNUSED;
		slots[i].format = FORMAT_AUTO;
		slots[i].color = 0xFFFFFFFF;
		strcpy(slots[i].fontName, "Verdana");
		for(int j=curParam; j < MAX_FIELDS; j++)
		{
			if(params[j].show)
			{
				slots[i].field = params[j].paramID;
				curParam = j + 1;
				break;
			}
		}
	}
}

void CSkin::RecalcLayout(int& width, int& height, BTN_PROPERTIES* m_properties)
{
	if(!(skinType & SKIN_BACKGOUND))
	{
		slotsRect.left = 0;
		for(int i=0; i < MAX_DIAGRAMS; i++)
		{
			if(diagrams[i].type != DIAGRAM_TYPE_HIDE)
			{
				diagrams[i].x = slotsRect.left;
				diagrams[i].y = 0;
				diagrams[i].height = height;
				slotsRect.left += diagrams[i].width;
			}
		}

		slotsRect.top = 0;
		slotsRect.bottom = height;
	} else
	{
		height = imgBackground.GetHeight();
		width = imgBackground.GetWidth();
	}


	int slotsUsed = 0;
	for(int i=0; i < MAX_SLOTS; i++)
	{
		if(slots[i].field != FIELDS_UNUSED)
		{
			slotsUsed++;
		}
		if(slots[i].hFont) 
		{
			DeleteObject(slots[i].hFont);
			slots[i].hFont = NULL;
		}
	}

	LOGFONT lf;
	ZeroMemory(&lf, sizeof(lf));
	if(slotsUsed)
	{
		lf.lfHeight = (slotsRect.bottom - slotsRect.top) / slotsUsed;
	} else
	{
		lf.lfHeight = 1;
	}
	lf.lfWeight = FW_NORMAL;
	lf.lfCharSet = ANSI_CHARSET;
	lf.lfOutPrecision = OUT_CHARACTER_PRECIS;
	lf.lfClipPrecision = CLIP_DEFAULT_PRECIS;
	lf.lfQuality = ANTIALIASED_QUALITY;
	lf.lfPitchAndFamily = FF_DONTCARE;
	strcpy(lf.lfFaceName, m_properties->fontName);
	//m_layout.txtFont =  CreateFontIndirect(&lf);

	int maxWidth = 0;
	HDC dc = GetDC(NULL);
	int y = slotsRect.top;
	for(int i=0; i < MAX_SLOTS; i++)
	{
		if(slots[i].field != FIELDS_UNUSED)
		{
			if(skinType & SKIN_AUTOSLOTS)
			{
				slots[i].height = (slotsRect.bottom - slotsRect.top) / slotsUsed;
				slots[i].x = slotsRect.left;
				slots[i].y = y;
				y += slots[i].height;
			}
			strcpy(lf.lfFaceName, slots[i].fontName);
			lf.lfHeight = slots[i].height;
			if(slots[i].bold)
			{
				lf.lfWeight = FW_BOLD;
			} else
			{
				lf.lfWeight = FW_NORMAL;
			}
			lf.lfItalic = slots[i].italic;
			slots[i].hFont = CreateFontIndirect(&lf);
			
			if(!(skinType & SKIN_BACKGOUND))
			{
				HFONT oldFont = (HFONT) SelectObject(dc, slots[i].hFont);
				char strVal[255];
				slots[i].GetFieldString(strVal, 0, TRUE);
				SIZE sz;
				GetTextExtentPoint32(dc, strVal, strlen(strVal), &sz);
				if(sz.cx > maxWidth)
				{
					maxWidth = sz.cx;
				}
				SelectObject(dc, oldFont);
			}
		}
	}
	ReleaseDC(NULL, dc);
	if(!(skinType & SKIN_BACKGOUND))
	{
		for(int i=0; i < MAX_SLOTS; i++)
		{
			if(slots[i].field != FIELDS_UNUSED)
			{
				slots[i].width = maxWidth;
			}
		}
		slotsRect.right = slotsRect.left + maxWidth;
		width = slotsRect.right;
	}
}

void CSkin::DrawDiagram(HDC dc, int x, int y, DIAGRAM* d, CONNECTION* connection)
{
	DWORD value = connection->GetFieldValue(d->field);
	switch(d->type)
	{
	case DIAGRAM_TYPE_HISTORY:
		{
			int width = 0;
			int dx = 0;
			int dy = 0;
			int startX = x;
			int startY = y;
			int dxValue = 0;
			int dyValue = 0;
			int maxPt = 0;

			int lineWidth = 1;
			if(d->fillType == HD_FILLTYPE_PICTURE && d->imgDiagram.IsValid())
			{
				if((d->startFrom & 0xF) == START_FROM_LEFT || (d->startFrom & 0xF) == START_FROM_RIGHT)
				{
					lineWidth = d->imgDiagram.GetWidth();
				} else
				{
					lineWidth = d->imgDiagram.GetHeight();
				}
			}

			switch(d->startFrom & 0xF)
			{
			case START_FROM_LEFT:
				maxPt = d->height;
				dx = 1;
				startX = d->x;
				dxValue = 0;
				if((d->startFrom & 0xF0) >> 4 == START_FROM_TOP)
				{
					dyValue = 1;
					dy = 0;
					startY = d->y;
				} else
				{
					dyValue = -1;
					startY = d->y + d->height - 1;
					dy = 0;
				}
				width = d->width;
				break;
			case START_FROM_RIGHT:
				maxPt = d->height;
				dx = -1;
				startX = d->x + d->width - 1;
				dxValue = 0;
				if((d->startFrom & 0xF0) >> 4 == START_FROM_TOP)
				{
					dyValue = 1;
					dy = 0;
					startY = d->y;
				} else
				{
					dyValue = -1;
					startY = d->y + d->height - 1;
					dy = 0;
				}
				width = d->width;
				break;
			case START_FROM_TOP:
				maxPt = d->width;
				dy = 1;
				startY = d->y;
				dyValue = 0;
				if((d->startFrom & 0xF0) >> 4 == START_FROM_LEFT)
				{
					dxValue = 1;
					startX = d->x;
					dx = 0;
				} else
				{
					dxValue = -1;
					startX = d->x + d->width - 1;
					dx = 0;
				}
				width = d->height;
				break;
			case START_FROM_BOTTOM:
				maxPt = d->width;
				dy = -1;
				startY = d->y + d->height - 1;
				dyValue = 0;
				if((d->startFrom & 0xF0) >> 4 == START_FROM_LEFT)
				{
					dxValue = 1;
					startX = d->x;
					dx = 0;
				} else
				{
					dxValue = -1;
					startX = d->x + d->width - 1;
					dx = 0;
				}
				width = d->height;
				break;
			}

			if(lineWidth)
			{
				width /= lineWidth;
			}

			DWORD maxValue = connection->GetFieldValue(FIELDS_BWLIMIT);
			if((connection->bwlType & 0xF) == CC_FLAG_BWAUTO)
			{
				maxValue = 0;
				for(int i=0; i < width && i < MAX_HISTORY; i++)
				{
					if(connection->in[i] > maxValue)
					{
						maxValue = connection->in[i];
					}
					if(connection->out[i] > maxValue)
					{
						maxValue = connection->out[i];
					}
				}
			}

			HPEN pen = CreatePen(PS_SOLID, 1, d->color);
			HPEN oldPen = (HPEN) SelectObject(dc, pen);

			LPDWORD history = 0;
			switch(d->field)
			{
			case FIELDS_INBANDWIDTH:
				history = connection->in;
				break;
			case FIELDS_OUTBANDWIDTH:
				history = connection->out;
				break;
			}

			if(history)
			{
				for(int i=0; i < width; i++)
				{
					switch(d->fillType)
					{ 
					case HD_FILLTYPE_SOLID:
						{
							MoveToEx(dc, x + startX, y + startY, NULL);
							int w =  maxValue == 0 ? 0 : (history[i] * maxPt / maxValue);
							if(w > maxPt) w = maxPt;
							LineTo(dc, x + startX + w * dxValue, y + startY + w * dyValue);
							startX += dx;
							startY += dy;
						}
						break;
					case HD_FILLTYPE_LINE:
						{
							int w =  maxValue == 0 ? 0 : (history[i] * maxPt / maxValue);
							if(w > maxPt) w = maxPt;
							if(i == 0)
							{
								MoveToEx(dc, x + startX + w * dxValue, y + startY + w * dyValue, NULL);
							} else
							{
								LineTo(dc, x + startX + w * dxValue, y + startY + w * dyValue);
							}
							startX += dx;
							startY += dy;
						}
						break;
					case HD_FILLTYPE_PICTURE:
						{
							switch(d->startFrom & 0xF)
							{
							case START_FROM_LEFT:
								{
									int height =  maxValue == 0 ? 0 : (history[i] * maxPt / maxValue);
									if(height > maxPt) height = maxPt;
									if(height)
									{
										switch((d->startFrom & 0xF0) >> 4)
										{
										case START_FROM_BOTTOM:
											{
												DrawVerticalImage(dc, x + startX, y + startY - height, height, &d->imgDiagram, &d->margins, d->tile, TRUE);
											}
											break;
										case START_FROM_TOP:
											{
												DrawVerticalImage(dc, x + startX, y + startY, height, &d->imgDiagram, &d->margins, d->tile, FALSE);
											}
											break;
										}
									}
								}
								break;
							case START_FROM_RIGHT:
								{
									int height =  maxValue == 0 ? 0 : (history[i] * maxPt / maxValue);
									if(height > maxPt) height = maxPt;
									if(height)
									{
										switch((d->startFrom & 0xF0) >> 4)
										{
										case START_FROM_BOTTOM:
											{
												DrawVerticalImage(dc, x + startX - d->imgDiagram.GetWidth(), y + startY - height, height, &d->imgDiagram, &d->margins, d->tile, TRUE);
											}
											break;
										case START_FROM_TOP:
											{
												DrawVerticalImage(dc, x + startX - d->imgDiagram.GetWidth(), y + startY, height, &d->imgDiagram, &d->margins, d->tile, FALSE);
											}
											break;
										}
									}
								}
								break;
							case START_FROM_TOP:
								{
									int width =  maxValue == 0 ? 0 : (history[i] * maxPt / maxValue);
									if(width > maxPt) width = maxPt;
									if(width)
									{
										switch((d->startFrom & 0xF0) >> 4)
										{
										case START_FROM_LEFT:
											{
												DrawHorizontalImage(dc, x + startX, y + startY, width, &d->imgDiagram, &d->margins, d->tile, TRUE);
											}
											break;
										case START_FROM_RIGHT:
											{
												DrawHorizontalImage(dc, x + startX - width, y + startY, width, &d->imgDiagram, &d->margins, d->tile, FALSE);
											}
											break;
										}
									}
								}
								break;
							case START_FROM_BOTTOM:
								{
									int width =  maxValue == 0 ? 0 : (history[i] * maxPt / maxValue);
									if(width > maxPt) width = maxPt;
									if(width)
									{
										switch((d->startFrom & 0xF0) >> 4)
										{
										case START_FROM_LEFT:
											{
												DrawHorizontalImage(dc, x + startX, y + startY - d->imgDiagram.GetHeight(), width, &d->imgDiagram, &d->margins, d->tile, TRUE);
											}
											break;
										case START_FROM_RIGHT:
											{
												DrawHorizontalImage(dc, x + startX - width, y + startY - d->imgDiagram.GetHeight(), width, &d->imgDiagram, &d->margins, d->tile, FALSE);
											}
											break;
										}
									}
								}
								break;
							}
							startX += dx * d->imgDiagram.GetWidth();
							startY += dy * d->imgDiagram.GetHeight();
						}
						break;
					}
				}
			}

			SelectObject(dc, oldPen);
			DeleteObject(pen);
		}
		break;
	case DIAGRAM_TYPE_LIGHT:
		{
			CxImage img;
			if(value)
			{
				d->imgDiagram.Crop(0, d->imgDiagram.GetHeight() / 2, d->imgDiagram.GetWidth(), d->imgDiagram.GetHeight(), &img);
			} else
			{
				d->imgDiagram.Crop(0, 0, d->imgDiagram.GetWidth(), d->imgDiagram.GetHeight() / 2, &img);
			}
			img.Draw(dc, x + d->x, y + d->y);
		}
		break;
	case DIAGRAM_TYPE_VERTICALROW:
		{
			if(value)
			{
				int fullHeight = d->height - d->margins.top - d->margins.bottom;
				int height = fullHeight * value / 100;
				if(height > fullHeight) height = fullHeight;
				if(d->margins.bottom)
				{
					CxImage bottom;
					d->imgDiagram.Crop(0, d->imgDiagram.GetHeight() - d->margins.bottom, d->width, d->imgDiagram.GetHeight(), &bottom);
					bottom.Draw(dc, x + d->x, y + d->y + d->height - d->margins.bottom);
				}
				CxImage center;
				d->imgDiagram.Crop(0, d->margins.top, d->width, d->imgDiagram.GetHeight() - d->margins.bottom, &center);
				long topY = 0;
				if(d->tile)
				{
					RECT rcTile;
					rcTile.left = x + d->x;
					rcTile.top = y + d->y + d->height - d->margins.bottom - height;
					rcTile.right = rcTile.left + center.GetWidth();
					rcTile.bottom = rcTile.top + height;
					int yy = rcTile.bottom - center.GetHeight();
					while(yy >= rcTile.top)
					{
						center.Draw(dc, rcTile.left, yy);
						topY = yy;
						yy -= center.GetHeight();
					}
				} else
				{
					center.Draw(dc, x + d->x, y + d->y + d->height - d->margins.bottom - height, -1, height);
					topY = y + d->y + d->height - d->margins.bottom - height - d->margins.top;
				}
				if(d->margins.top)
				{
					CxImage top;
					d->imgDiagram.Crop(0, 0, d->width, d->margins.top, &top);
					top.Draw(dc, x + d->x, topY);
				}
			}
		}
		break;
	case DIAGRAM_TYPE_HORIZONTALROW:
		{
			if(value)
			{
				int fullWidth = d->width - d->margins.left - d->margins.right;
				int width = fullWidth * value / 100;
				if(width > fullWidth) width = fullWidth;
				if(d->margins.left)
				{
					CxImage right;
					d->imgDiagram.Crop(0, 0, d->margins.left, d->imgDiagram.GetHeight(), &right);
					right.Draw(dc, x + d->x, d->y);
				}
				CxImage center;
				d->imgDiagram.Crop(d->margins.left, 0, d->imgDiagram.GetWidth() - d->margins.right, d->imgDiagram.GetHeight(), &center);
				long rightX = 0;
				if(d->tile)
				{
					RECT rcTile;
					rcTile.top = y + d->y;
					rcTile.left = x + d->x + d->margins.left;
					rcTile.right = rcTile.left + width - d->margins.right;
					rcTile.bottom = rcTile.top + center.GetHeight();
					int xx = rcTile.left;
					while(xx + (int) center.GetWidth() <= rcTile.right)
					{
						center.Draw(dc, xx, rcTile.top);
						rightX = xx;
						xx += center.GetWidth();
					}
				} else
				{
					center.Draw(dc, x + d->x + d->margins.left, y + d->y, -1, width);
					rightX = x + d->width - d->margins.right - width - d->margins.left;
				}
				if(d->margins.right)
				{
					CxImage right;
					d->imgDiagram.Crop(d->width - d->margins.right, 0, d->width, d->imgDiagram.GetWidth(), &right);
					right.Draw(dc, rightX , 0);
				}
			}
		}
		break;
	}
}

void CSkin::Clear(void)
{
	for(int i=0; i < MAX_DIAGRAMS; i++)
	{
		diagrams[i].type = DIAGRAM_TYPE_HIDE;
		diagrams[i].imgDiagram.Destroy();
	}

	for(int i=0; i < MAX_SLOTS; i++)
	{
		if(slots[i].hFont)
		{
			DeleteObject(slots[i].hFont);
		}
		slots[i].field = FIELDS_UNUSED;
		slots[i].format = FORMAT_AUTO;
	}
	ZeroMemory(slots, sizeof(slots));
}

void CSkin::ParsePoint(char* str, int& x, int& y)
{
	char strVal[50];
	char* curPos = str;
	while(curPos[0] && !isdigit(curPos[0])) curPos++;
	int i=0;
	while(curPos[0] && isdigit(curPos[0])) 
	{
		strVal[i] = curPos[0];
		i++;
		curPos++;
	}
	strVal[i] = 0;
	x = atoi(strVal);

	while(curPos[0] && !isdigit(curPos[0])) curPos++;

	i=0;
	while(curPos[0] && isdigit(curPos[0])) 
	{
		strVal[i] = curPos[0];
		i++;
		curPos++;
	}
	strVal[i] = 0;
	y = atoi(strVal);
}

COLORREF CSkin::ParseColor(char* str)
{
	DWORD r = 0;
	DWORD g = 0;
	DWORD b = 0;
	char strVal[50];
	char* curPos = str;
	while(curPos[0] && !isdigit(curPos[0])) curPos++;
	int i=0;
	while(curPos[0] && isdigit(curPos[0])) 
	{
		strVal[i] = curPos[0];
		i++;
		curPos++;
	}
	strVal[i] = 0;
	r = atoi(strVal);

	while(curPos[0] && !isdigit(curPos[0])) curPos++;

	i=0;
	while(curPos[0] && isdigit(curPos[0])) 
	{
		strVal[i] = curPos[0];
		i++;
		curPos++;
	}
	strVal[i] = 0;
	g = atoi(strVal);

	while(curPos[0] && !isdigit(curPos[0])) curPos++;

	i=0;
	while(curPos[0] && isdigit(curPos[0])) 
	{
		strVal[i] = curPos[0];
		i++;
		curPos++;
	}
	strVal[i] = 0;
	b = atoi(strVal);

	return RGB(r, g, b);
}

void CSkin::DrawVerticalImage(HDC dc, int x, int y, int height, CxImage* img, LPRECT margins, BOOL tile, BOOL bottomAlign)
{
	if(margins->bottom)
	{
		CxImage bottom;
		img->Crop(0, img->GetHeight() - margins->bottom, img->GetWidth(), img->GetHeight(), &bottom);
		bottom.Draw(dc, x, y + height - margins->bottom);
	}
	CxImage center;
	img->Crop(0, margins->top, img->GetWidth(), img->GetHeight() - margins->bottom, &center);
	long topY = 0;
	if(tile)
	{
		if(bottomAlign)
		{
			RECT rcTile;
			rcTile.left = x;
			rcTile.top = y;// - margins->bottom - height;
			rcTile.right = rcTile.left + center.GetWidth();
			rcTile.bottom = rcTile.top + height;
			int yy = rcTile.bottom - center.GetHeight();
			while(yy >= rcTile.top)
			{
				center.Draw(dc, rcTile.left, yy);
				topY = yy;
				yy -= center.GetHeight();
			}
		} else
		{
			RECT rcTile;
			rcTile.left = x;
			rcTile.top = y;// - margins->bottom - height;
			rcTile.right = rcTile.left + center.GetWidth();
			rcTile.bottom = rcTile.top + height;
			int yy = rcTile.top;
			topY = yy;
			while(yy <= rcTile.bottom)
			{
				center.Draw(dc, rcTile.left, yy);
				//topY = yy;
				yy += center.GetHeight();
			}
		}
	} else
	{
		center.Draw(dc, x, y + height - margins->bottom, -1, height);
		topY = y + height - margins->bottom - margins->top;
	}
	if(margins->top)
	{
		CxImage top;
		img->Crop(0, 0, img->GetWidth(), margins->top, &top);
		top.Draw(dc, x, topY);
	}
}

void CSkin::DrawHorizontalImage(HDC dc, int x, int y, int width, CxImage* img, LPRECT margins, BOOL tile, BOOL leftAlign)
{
	if(margins->left)
	{
		CxImage right;
		img->Crop(0, 0, margins->left, img->GetHeight(), &right);
		right.Draw(dc, x, y);
	}
	CxImage center;
	img->Crop(margins->left, 0, img->GetWidth() - margins->right, img->GetHeight(), &center);
	long rightX = 0;
	if(tile)
	{
		if(leftAlign)
		{
			RECT rcTile;
			rcTile.top = y;
			rcTile.left = x + margins->left;
			rcTile.right = rcTile.left - margins->right + width;
			rcTile.bottom = rcTile.top + center.GetHeight();
			int xx = rcTile.left;
			while(xx + (int) center.GetWidth() <= rcTile.right)
			{
				center.Draw(dc, xx, rcTile.top);
				rightX = xx;
				xx += center.GetWidth();
			}
		} else
		{
			RECT rcTile;
			rcTile.top = y;
			rcTile.left = x + margins->left;
			rcTile.right = rcTile.left - margins->right + width;
			rcTile.bottom = rcTile.top + center.GetHeight();
			int xx = rcTile.right - center.GetWidth();
			rightX = xx;
			while(xx - (int) center.GetWidth() >= rcTile.left)
			{
				center.Draw(dc, xx, rcTile.top);
				xx -= center.GetWidth();
			}
		}
	} else
	{
		center.Draw(dc, x + + margins->left, y, -1, width);
		rightX = x - margins->right - width - margins->left;
	}
	if(margins->right)
	{
		CxImage right;
		img->Crop(width - margins->right, 0, width, img->GetWidth(), &right);
		right.Draw(dc, rightX , 0);
	}
}
