// NetMonBtn.h: interface for the CNetMonBtn class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_NetMonBtn_H__925A811E_357A_4F85_931D_D348ED087B66__INCLUDED_)
#define AFX_NetMonBtn_H__925A811E_357A_4F85_931D_D348ED087B66__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "arrays.h"

#define MAX_HISTORY		900

struct CONNECTION
{
	WCHAR		idName[MAX_ADAPTER_NAME];
	char		name[256];
	MIB_IFROW	prevInfo;
	DWORD 		prevUpdateTime;
	MIB_IFROW	connectionInfo;
	DWORD 		updateTime;
	char		ipAddress[16];
	time_t		beginTime;

	DWORD		bwlType;
	ULONG		bwLimit;

	DWORD		in[MAX_HISTORY];
	DWORD		out[MAX_HISTORY];

	BOOL		exists;

	int			x;
	int			y;
	int			width;
	int			height;

	DWORD GetFieldValue(DWORD fieldID);
};

#define FORMAT_AUTO			0x0000
#define FORMAT_BYTE			0x1000
#define FORMAT_KB 			0x2000
#define FORMAT_MB 			0x3000

#define MAX_SLOTS			10

#define ALIGN_RIGHT			0
#define ALIGN_LEFT			1
#define ALIGN_CENTER		2

struct SLOT
{
	DWORD		field;
	DWORD		format;
	COLORREF	color;
	BOOL		label;

	int			x;
	int			y;
	int			width;
	int			height;
	HFONT		hFont;

	char		fontName[255];
	BOOL		bold;
	BOOL		italic;
	DWORD		align;

	void GetFieldString(char* strValue, DWORD value, BOOL maxString);
};

struct LAYOUT
{
	int			rows;
	int			cols;
	int			itemWidth;
	int			itemHeight;

	int			x;
	int			y;
};

#define SKIN_BACKGOUND	0x01
#define SKIN_AUTOSLOTS	0x02

#define DIAGRAM_INBOUND		0x0001
#define DIAGRAM_OUTBOUND	0x0002
#define DIAGRAM_TOTAL		0x0004
#define DIAGRAM_HORIZONTAL	0x1000

#define DIAGRAM_TYPE_HIDE			0
#define DIAGRAM_TYPE_VERTICALROW	1
#define DIAGRAM_TYPE_HORIZONTALROW	2
#define DIAGRAM_TYPE_LIGHT			3
#define DIAGRAM_TYPE_PERCENT		4
#define DIAGRAM_TYPE_HISTORY		5

#define START_FROM_RIGHT			0
#define START_FROM_LEFT				1
#define START_FROM_TOP				2
#define START_FROM_BOTTOM			3

#define HD_FILLTYPE_SOLID			0
#define HD_FILLTYPE_LINE			1
#define HD_FILLTYPE_PICTURE			2

struct DIAGRAM
{
	DWORD	type;
	DWORD	field;

	COLORREF color;
	DWORD    startFrom;
    DWORD	 fillType;

	CxImage imgDiagram;
	RECT	margins;
	BOOL	tile;

	int		x;
	int		y;
	int		width;
	int		height;
};

class CNetMonBtn;

#define MAX_DIAGRAMS		6

class CSkin
{
public:
	DWORD	skinType;
	
	CxImage	imgBackground;

	DIAGRAM diagrams[MAX_DIAGRAMS];

	RECT	slotsRect;
	SLOT	slots[MAX_SLOTS];

	CSkin();
	~CSkin();

	void Load(LPSTR fileName, PARAMS* params);
	void Draw(HDC dc, int x, int y, CONNECTION* conn);
	void LoadDefault(PARAMS* params);
	void RecalcLayout(int& width, int& height, BTN_PROPERTIES* m_properties);
	void DrawDiagram(HDC dc, int x, int y, DIAGRAM* d, CONNECTION* connection);
	void Clear(void);
	void ParsePoint(char* str, int& x, int& y);
	COLORREF ParseColor(char* str);
private:
	void DrawVerticalImage(HDC dc, int x, int y, int height, CxImage* img, LPRECT margins, BOOL tile, BOOL bottomAlign);
public:
	void DrawHorizontalImage(HDC dc, int x, int y, int width, CxImage* img, LPRECT margins, BOOL tile, BOOL leftAlign);
};

#define CC_FLAG_HIDE		0x80
#define CC_FLAG_BWDEFAULT	0x00
#define CC_FLAG_BWAUTO		0x01
#define CC_FLAG_BWCUSTOM	0x02

struct CUSTOM_CONNECTION
{
	WCHAR id[300];
	char  displayName[300];
	DWORD flags;
	ULONG bandWidthLimit;
};

class CPropertiesPP;

class CNetMonBtn : public ITlbButton  
{
	HMODULE m_hResource;

	HICON	m_icModem;
	HICON	m_icNetcard;

	DWORD m_version;
	char m_Name[MAX_PATH];
	DWORD m_state;
	DWORD m_ID;
	RECT  m_rc;
	ITlbContainer* m_container;

	CTArray<CONNECTION> m_connections;
	
	CTArray<CUSTOM_CONNECTION> m_customConnections;

	LAYOUT			m_layout;
	CSkin			m_skin;
	char			m_skinPath[MAX_PATH];
	char			m_run[MAX_PATH * 2];

	int m_overConnection;

	HANDLE hStop;
	HANDLE hTimerThread;
	static DWORD WINAPI TimerTrdProc(LPVOID lpParam);
	void TimerThread();
	void StartTimer();

	int		m_rows;
	int		m_cols;

	char    m_onConnectSound[MAX_PATH];
	char    m_onDisconnectSound[MAX_PATH];

	CPropertiesPP* m_propDlg;

	CRITICAL_SECTION m_sync;

	void LockItems() { EnterCriticalSection(&m_sync); }
	void UnlockItems() { LeaveCriticalSection(&m_sync); }

protected:
	DWORD m_ObjRefCount;
public:
	BTN_PROPERTIES	m_properties;
	PARAMS			m_params[MAX_FIELDS];

	void RecalcLayout(int height);
	CNetMonBtn();
	virtual ~CNetMonBtn();

	//IUnknown methods
	STDMETHODIMP QueryInterface(REFIID, LPVOID*);
	STDMETHODIMP_(DWORD) AddRef();
	STDMETHODIMP_(DWORD) Release();

	//ITlbButton methods
    STDMETHODIMP Load(IStream *data);
    STDMETHODIMP Save(IStream *data);
    STDMETHODIMP SetRect(RECT *rcItem);
    STDMETHODIMP SetContainer(ITlbContainer *cont);
    STDMETHODIMP SetID(int btnID);
    STDMETHODIMP Draw(HANDLE hDC, RECT *rcItem);
    STDMETHODIMP OnMouseOver(long keys, int x, int y);
    STDMETHODIMP OnMouseLeave(long keys);
    STDMETHODIMP OnMouseEnter(long keys, int x, int y);
	STDMETHODIMP Create(IStream* data);
	STDMETHODIMP StateChanged(int state, int* needRedraw);
	STDMETHODIMP GetDescription(char* szDescription);
	STDMETHODIMP OnLButtonDown(long key, int x, int y);
	STDMETHODIMP OnLButtonUp(long key, int x, int y);
	STDMETHODIMP Run();
	STDMETHODIMP DoAction(int action);
	STDMETHODIMP SetName(LPCSTR szName);
	STDMETHODIMP_(int) QueryContextMenu(HMENU hMenu, int index, int cmdFirst, int cmdLast);
	STDMETHODIMP InvokeCommand(int ID);
	STDMETHODIMP SetLCID(DWORD dwLCID, HMODULE hResModule);
	STDMETHODIMP GetIconLocation(LPSTR szLocation);
	STDMETHODIMP SetIconLocation(LPCSTR szLocation);
	STDMETHODIMP GetButtonInfo(BUTTONINFO* btninfo);
	STDMETHODIMP Invoke(DWORD id, WPARAM wParam, LPARAM lParam);
private:
	DWORD LoadConnections(void);
};

#endif // !defined(AFX_NetMonBtn_H__925A811E_357A_4F85_931D_D348ED087B66__INCLUDED_)
