#include "globals.h"
#include "propertiesdlg.h"
#include <commctrl.h>
#include <stdio.h>
#include "resource.h"
#include <Commdlg.h>

void Version2Str(DWORD versionMS, DWORD versionLS, char* str)
{
	DWORD ver[4];
	ver[0] = HIWORD(versionMS);
	ver[1] = LOWORD(versionMS);
	ver[2] = HIWORD(versionLS);
	ver[3] = LOWORD(versionLS);
	int lastNum = 3;
	for(int i=3; i >=0; i--)
	{
		if(ver[i])
		{
			lastNum = i;
			break;
		}
	}
	str[0] = 0;
	for(int i=0; i <= lastNum; i++)
	{
		char num[100];
		itoa(ver[i], num, 10);
		strcat(str, num);
		if(i != lastNum)
		{
			strcat(str, ".");
		}
	}
}

void GetFileVersion(LPCTSTR fileName, DWORD& versionMS, DWORD& versionLS)
{
	versionMS = 0;
	versionLS = 0;
	DWORD dw;
	DWORD verSize = GetFileVersionInfoSize((LPTSTR) fileName, &dw);
	if(verSize)
	{
		void* verData = malloc(verSize);
		if(GetFileVersionInfo((LPTSTR) fileName, NULL, verSize, verData))
		{
			VS_FIXEDFILEINFO* verInfo;
			UINT len = 0;
			if(VerQueryValue(verData, TEXT("\\"), (void**) &verInfo, &len))
			{
				versionMS = verInfo->dwFileVersionMS;
				versionLS = verInfo->dwFileVersionLS;
			}
		}
		free(verData);
	}
}

CPropertiesDlg::CPropertiesDlg(void) : CTrueDialog(IDD_PROPERTIES)
{
	ZeroMemory(&m_sampleData, sizeof(m_sampleData));
	srand( (unsigned)time( NULL ) );
	for(int i=0; i < MAX_HISTORY; i++)
	{
		m_sampleData.in[i] = rand();
		m_sampleData.out[i] = rand();
	}
	m_sampleData.bwlType = CC_FLAG_BWAUTO;
	m_sampleData.bwLimit = 10000000 / 8;
	lockInit = TRUE;
	m_sampleData.updateTime = 2000;
	m_sampleData.prevUpdateTime = 1000;
	ZeroMemory(&m_sampleData.connectionInfo, sizeof(m_sampleData.connectionInfo));
	m_sampleData.connectionInfo.dwType = MIB_IF_TYPE_ETHERNET;
	m_sampleData.connectionInfo.dwSpeed = 10000000;
	m_sampleData.connectionInfo.dwOperStatus = MIB_IF_OPER_STATUS_OPERATIONAL;
	m_sampleData.connectionInfo.dwLastChange = 20000;
	m_sampleData.connectionInfo.dwInOctets  = 245678192;
	m_sampleData.connectionInfo.dwOutOctets = 7839282;
	m_sampleData.prevInfo = m_sampleData.connectionInfo;
	strcpy(m_sampleData.name, "Connection");

	m_sampleData.prevInfo.dwInOctets  = m_sampleData.connectionInfo.dwInOctets - 400000;
	m_sampleData.prevInfo.dwOutOctets = m_sampleData.connectionInfo.dwOutOctets - 600000;

	strcpy(m_sampleData.ipAddress, "194.367.27.19");
}

CPropertiesDlg::~CPropertiesDlg(void)
{
}

int CALLBACK MyEnumFonts(ENUMLOGFONTEX *lpelfe, NEWTEXTMETRICEX *lpntme, DWORD FontType, LPARAM lParam)
{
	HWND hwndCB = (HWND) lParam;
	if(SendMessage(hwndCB, CB_FINDSTRING, NULL, (LPARAM) lpelfe->elfFullName) == CB_ERR)
	{
		SendMessage(hwndCB, CB_ADDSTRING, NULL, (LPARAM) lpelfe->elfFullName);
	}
	return TRUE;
}

void CPropertiesDlg::OnInitDialog(void)
{
	FindSkins();

	SetDlgItemText(m_hWnd, IDC_RUN, m_parent->m_run);
	SetDlgItemText(m_hWnd, IDC_CONNECTFILE, m_parent->m_onConnectSound);
	SetDlgItemText(m_hWnd, IDC_DISCONNECTFILE, m_parent->m_onDisconnectSound);

	SendDlgItemMessage(m_hWnd, IDC_ROWSPIN, UDM_SETRANGE, 0, (LPARAM) MAKELONG((short) 20, (short) 1));
	SendDlgItemMessage(m_hWnd, IDC_ROWSPIN, UDM_SETPOS, 0, (LPARAM) MAKELONG((short) m_parent->m_props.rows, (short) 0));

	HWND hList = GetDlgItem(m_hWnd, IDC_PARAMS);
	ListView_SetExtendedListViewStyleEx(hList, LVS_EX_CHECKBOXES | LVS_EX_FULLROWSELECT | LVS_EX_INFOTIP, 
		LVS_EX_CHECKBOXES | LVS_EX_FULLROWSELECT | LVS_EX_INFOTIP);

	RECT rcList;
	GetClientRect(hList, &rcList);

	LVCOLUMN lvc;
	ZeroMemory(&lvc, sizeof(lvc));
	lvc.mask = LVCF_WIDTH;
	lvc.cx = rcList.right - rcList.left - GetSystemMetrics(SM_CXVSCROLL);
	ListView_InsertColumn(hList, 0, &lvc);

	for(int i=0; i < MAX_FIELDS; i++)
	{
		char txtName[255];
		strcpy(txtName, "???");
		for(int j=0; j < MAX_FIELDS; j++)
		{
			if(m_parent->m_params[i].paramID == fieldsNames[j].fieldID)
			{
				LoadString(m_parent->m_hResource, fieldsNames[j].strID, txtName, 255);
			}
		}

		LVITEM lvi;
		ZeroMemory(&lvi, sizeof(lvi));
		lvi.mask = LVIF_PARAM | LVIF_TEXT;
		lvi.iItem = i;
		lvi.lParam = m_parent->m_params[i].paramID;
		lvi.pszText = txtName;
		ListView_InsertItem(hList, &lvi);
		if(m_parent->m_params[i].show)
		{
			ListView_SetCheckState(hList, i, TRUE);
		} else
		{
			ListView_SetCheckState(hList, i, FALSE);
		}
	}

	lockInit = FALSE;
	InitSkin();
}

void CPropertiesDlg::OnCommand(DWORD itemID)
{
	switch(LOWORD(itemID))
	{
	case IDC_CONNECTSOUND:
	case IDC_DISCONNECTSOUND:
		{
			char fileName[500];
			fileName[0] = 0;
			OPENFILENAME ofn;
			memset(&ofn, 0, sizeof(ofn));
			ofn.lStructSize = sizeof(ofn);
			ofn.hwndOwner = m_hWnd;
			ofn.hInstance = g_hInst;
			ofn.lpstrFilter = "Sound files\0*.wav\0";
			ofn.lpstrCustomFilter = NULL;
			ofn.lpstrFile = fileName;
			ofn.nMaxFile = 500;
			ofn.lpstrTitle = "Select sound file";
			ofn.Flags = OFN_ENABLESIZING | OFN_FILEMUSTEXIST | OFN_LONGNAMES | OFN_PATHMUSTEXIST;
			
			if(GetOpenFileName(&ofn))
			{
				if(LOWORD(itemID) == IDC_CONNECTSOUND)
				{
					SetDlgItemText(m_hWnd, IDC_CONNECTFILE, fileName);
				} else
				{
					SetDlgItemText(m_hWnd, IDC_DISCONNECTFILE, fileName);
				}
			}
		}
		break;
	case IDC_OPENCOMMAND:
		{
			char fileName[500];
			fileName[0] = 0;
			OPENFILENAME ofn;
			memset(&ofn, 0, sizeof(ofn));
			ofn.lStructSize = sizeof(ofn);
			ofn.hwndOwner = m_hWnd;
			ofn.hInstance = g_hInst;
			ofn.lpstrFilter = "All files\0*.*\0";
			ofn.lpstrCustomFilter = NULL;
			ofn.lpstrFile = fileName;
			ofn.nMaxFile = 500;
			ofn.lpstrTitle = "Select the file to run on click";
			ofn.Flags = OFN_ENABLESIZING | OFN_FILEMUSTEXIST | OFN_LONGNAMES | OFN_PATHMUSTEXIST;
			
			if(GetOpenFileName(&ofn))
			{
				SetDlgItemText(m_hWnd, IDC_RUN, fileName);
			}
		}
		break;
	case IDC_SKIN:
		if(HIWORD(itemID) == CBN_SELCHANGE)
		{
			InitSkin();
			RedrawPreview();
		}
		break;
	case IDC_UP:
		{
			char txt[255];
			HWND hList = GetDlgItem(m_hWnd, IDC_PARAMS);

			for(int i=0; i < MAX_FIELDS; i++)
			{
				LVITEM lvi;
				ZeroMemory(&lvi, sizeof(lvi));
				lvi.iItem = i;
				lvi.mask = LVIF_PARAM | LVIF_TEXT | LVIF_STATE;
				lvi.pszText = txt;
				lvi.cchTextMax = 255;
				lvi.stateMask = LVIS_SELECTED;
				ListView_GetItem(hList, &lvi);

                if(lvi.state == LVIS_SELECTED)
				{
					if(i != 0)
					{
						lockInit = TRUE;
						int show = ListView_GetCheckState(hList, i);
						ListView_DeleteItem(hList, i);
						lvi.iItem = i - 1;
						int idx = ListView_InsertItem(hList, &lvi);
						ListView_SetCheckState(hList, idx, show);
						ListView_EnsureVisible(hList, idx, FALSE);
						lockInit = FALSE;

						InitSkin();
						RedrawPreview();
					}
					break;
                }
			}
		}
		break;
	case IDC_DOWN:
		{
			char txt[255];
			HWND hList = GetDlgItem(m_hWnd, IDC_PARAMS);

			for(int i=0; i < MAX_FIELDS; i++)
			{
				LVITEM lvi;
				ZeroMemory(&lvi, sizeof(lvi));
				lvi.iItem = i;
				lvi.mask = LVIF_PARAM | LVIF_TEXT | LVIF_STATE;
				lvi.pszText = txt;
				lvi.cchTextMax = 255;
				lvi.stateMask = LVIS_SELECTED;
				ListView_GetItem(hList, &lvi);

                if(lvi.state == LVIS_SELECTED)
				{
					lockInit = TRUE;
					int show = ListView_GetCheckState(hList, i);
					ListView_DeleteItem(hList, i);
					lvi.iItem = i + 1;
					int idx = ListView_InsertItem(hList, &lvi);
					ListView_SetCheckState(hList, idx, show);
					ListView_EnsureVisible(hList, idx, FALSE);
					lockInit = FALSE;

					InitSkin();
					RedrawPreview();
					break;
                }
			}
		}
		break;
	case IDC_GETSKINS:
		ShellExecute(NULL, "open", "http://www.truelaunchbar.com/skinslib/index.php?show=5", NULL, NULL, SW_SHOWNORMAL);
		break;
	}
}

void CPropertiesDlg::OnPaint(void)
{
	PAINTSTRUCT ps;
	BeginPaint(m_hWnd, &ps);
	
	RECT rcPreview;
	GetWindowRect(GetDlgItem(m_hWnd, IDC_TEST), &rcPreview);
	MapWindowPoints(NULL, m_hWnd, (LPPOINT) &rcPreview, 2);

	int oldTextMode = SetBkMode(ps.hdc, TRANSPARENT);

	m_skin.Draw(ps.hdc, rcPreview.left, rcPreview.top, &m_sampleData);

	SetBkMode(ps.hdc, oldTextMode);

	EndPaint(m_hWnd, &ps);
}

void CPropertiesDlg::RepaintButton(void)
{
	RECT rcButton;
	GetWindowRect(GetDlgItem(m_hWnd, IDC_TEST), &rcButton);
	MapWindowPoints(NULL, m_hWnd, (LPPOINT) &rcButton, 2);
	InvalidateRect(m_hWnd, &rcButton, TRUE);
}


void CPropertiesDlg::FindSkins(void)
{
	int idx = SendDlgItemMessage(m_hWnd, IDC_SKIN, CB_ADDSTRING, 0, (LPARAM) "<default>");
	SendDlgItemMessage(m_hWnd, IDC_SKIN, CB_SETITEMDATA, idx, -1);
	char path[MAX_PATH];
	GetModuleFileName(g_hInst, path, MAX_PATH);
	PathRemoveFileSpec(path);
	PathAddBackslash(path);
	strcat(path, "skins\\");

	WIN32_FIND_DATA fd;
	char findMask[MAX_PATH];
	strcpy(findMask, path);
	strcat(findMask, "*");

	HANDLE hFind = FindFirstFile(findMask, &fd);
	if(hFind != INVALID_HANDLE_VALUE)
	{
		do 
		{
			if(strcmp(fd.cFileName, ".") && strcmp(fd.cFileName, "..") && (fd.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY))
			{
				char SkinName[255];
				char skinFile[MAX_PATH];
				strcpy(skinFile, path);
				strcat(skinFile, fd.cFileName);
				PathAddBackslash(skinFile);
				strcat(skinFile, "netmonskin.ini");

				GetPrivateProfileString("description", "SkinName", "", SkinName, 255, skinFile);
				if(SkinName[0])
				{
					SKINS skin;
					strcpy(skin.path, skinFile);
					int idx = SendDlgItemMessage(m_hWnd, IDC_SKIN, CB_ADDSTRING, 0, (LPARAM) SkinName);
					SendDlgItemMessage(m_hWnd, IDC_SKIN, CB_SETITEMDATA, idx, m_skins.GetCount());
					m_skins.Add(&skin);
				}
			}
		} 
		while(FindNextFile(hFind, &fd));
		FindClose(hFind);
	}

	SetCBData(IDC_SKIN, -1);
	char skinPath[MAX_PATH];
	GetFullSkinPath(m_parent->m_skinPath, skinPath);

	for(int i=0; i < m_skins.GetCount(); i++)
	{
		if(!StrCmpI(m_skins[i].path, skinPath))
		{
			SetCBData(IDC_SKIN, i);
		}
	}
}

DWORD CPropertiesDlg::GetCBData(DWORD cbID)
{
	int idx = SendDlgItemMessage(m_hWnd, cbID, CB_GETCURSEL, 0, 0);
	return SendDlgItemMessage(m_hWnd, cbID, CB_GETITEMDATA, idx, 0);
}

void CPropertiesDlg::SetCBData(DWORD cbID, int data)
{
	int cbCount = SendDlgItemMessage(m_hWnd, cbID, CB_GETCOUNT, 0, 0);
	for(int i=0; i < cbCount; i++)
	{
		int dt = SendDlgItemMessage(m_hWnd, cbID, CB_GETITEMDATA, i, 0);
		if(dt == data)
		{
			SendDlgItemMessage(m_hWnd, cbID, CB_SETCURSEL, i, 0);
			break;
		}
	}
}

void CPropertiesDlg::RedrawPreview(void)
{
	RECT rcPreview;
	GetWindowRect(GetDlgItem(m_hWnd, IDC_TEST), &rcPreview);
	MapWindowPoints(NULL, m_hWnd, (LPPOINT) &rcPreview, 2);

	InvalidateRect(m_hWnd, &rcPreview, TRUE);
}

void CPropertiesDlg::InitSkin(void)
{
	if(lockInit) return;
	GetProperties();

	RECT rcPreview;
	GetWindowRect(GetDlgItem(m_hWnd, IDC_TEST), &rcPreview);

	char curSkin[MAX_PATH];
	curSkin[0] = 0;
	int skinIDX = GetCBData(IDC_SKIN);
	if(skinIDX >=0 && skinIDX < m_skins.GetCount())
	{
		GetRelSkinPath(m_skins[skinIDX].path, curSkin);
	}
	m_skin.Load(curSkin, m_parent->m_params);

	int width = rcPreview.right - rcPreview.left;
	int height = rcPreview.bottom - rcPreview.top;

	m_skin.RecalcLayout(width, height, &m_parent->m_props);
}

void CPropertiesDlg::GetProperties(void)
{
	m_parent->m_props.rows = 0xFFFF & SendDlgItemMessage(m_hWnd, IDC_ROWSPIN, UDM_GETPOS, 0, 0);
	if(!m_parent->m_props.rows) m_parent->m_props.rows = 1;

	int skinIDX = GetCBData(IDC_SKIN);
	if(skinIDX >=0 && skinIDX < m_skins.GetCount())
	{
		GetRelSkinPath(m_skins[skinIDX].path, m_parent->m_skinPath);
	} else
	{
		m_parent->m_skinPath[0] = 0;
	}

	HWND hList = GetDlgItem(m_hWnd, IDC_PARAMS);

	for(int i=0; i < MAX_FIELDS; i++)
	{
		LVITEM lvi;
		ZeroMemory(&lvi, sizeof(lvi));
		lvi.mask = LVIF_PARAM;
		lvi.iItem = i;

		ListView_GetItem(hList, &lvi);
		m_parent->m_params[i].paramID = (DWORD) lvi.lParam;

		if(ListView_GetCheckState(hList, i))
		{
			m_parent->m_params[i].show = TRUE;
		} else
		{
			m_parent->m_params[i].show = FALSE;
		}
	}
}

DWORD CPropertiesDlg::OnNotify(int idCtrl, LPNMHDR pnmh)
{
	switch(pnmh->code)
	{
	case PSN_APPLY:
		{
			GetDlgItemText(m_hWnd, IDC_RUN, m_parent->m_run, MAX_PATH * 2);
			GetDlgItemText(m_hWnd, IDC_CONNECTFILE, m_parent->m_onConnectSound, MAX_PATH);
			GetDlgItemText(m_hWnd, IDC_DISCONNECTFILE, m_parent->m_onDisconnectSound, MAX_PATH);

			GetProperties();
		}
		break;
	case PSN_HELP:
		break;
	}

	switch(idCtrl)
	{
	case IDC_PARAMS:
		{
			LPNMLISTVIEW pnmv = (LPNMLISTVIEW) pnmh;
			if(pnmv->hdr.code == LVN_ITEMCHANGED)
			{
				if(!lockInit)
				{
					InitSkin();
					RedrawPreview();
				}
			}
		}
	}
	return TRUE;
}

CCustConnectionsDlg::CCustConnectionsDlg(void) : CTrueDialog(IDD_CONNECTIONS)
{
	m_lockUpdate = FALSE;
}

CCustConnectionsDlg::~CCustConnectionsDlg(void)
{
}

CPropertiesPP::CPropertiesPP(void)
{
	m_hWnd = NULL;
}

CPropertiesPP::~CPropertiesPP(void)
{
}

CAboutDlg::CAboutDlg(void) : CTrueDialog(IDD_ABOUT)
{

}

CAboutDlg::~CAboutDlg(void)
{
}


DWORD CPropertiesPP::Open(HWND hWnd, HMODULE hResModule)
{
	if(m_hWnd)
	{
		SetActiveWindow(m_hWnd);
		return 0;
	}

	m_hResource = hResModule;

	m_general.m_parent	= this;
	m_custCon.m_parent	= this;
	m_about.m_parent	= this;

	PROPSHEETPAGE psp[3];
	PROPSHEETHEADER psh;

	psp[0].dwSize = sizeof(PROPSHEETPAGE);
	psp[0].dwFlags = PSP_USETITLE | PSP_DLGINDIRECT;
	psp[0].hInstance = m_hResource;
	psp[0].pResource = m_general.GetTemplate(m_hResource);
	psp[0].pszIcon = NULL;
	psp[0].pfnDlgProc = (DLGPROC) CTrueDialog::WndProcPP;
	psp[0].pszTitle = MAKEINTRESOURCE(IDS_GENERAL);
	psp[0].lParam = (LPARAM) &m_general;
	psp[0].pfnCallback = NULL;

	psp[1].dwSize = sizeof(PROPSHEETPAGE);
	psp[1].dwFlags = PSP_USETITLE | PSP_DLGINDIRECT;
	psp[1].hInstance = m_hResource;
	psp[1].pResource = m_custCon.GetTemplate(m_hResource);
	psp[1].pszIcon = NULL;
	psp[1].pfnDlgProc = (DLGPROC) CTrueDialog::WndProcPP;
	psp[1].pszTitle = MAKEINTRESOURCE(IDS_CUSCON);
	psp[1].lParam = (LPARAM) &m_custCon;
	psp[1].pfnCallback = NULL;

	psp[2].dwSize = sizeof(PROPSHEETPAGE);
	psp[2].dwFlags = PSP_USETITLE | PSP_DLGINDIRECT;
	psp[2].hInstance = m_hResource;
	psp[2].pResource = m_about.GetTemplate(m_hResource);
	psp[2].pszIcon = NULL;
	psp[2].pfnDlgProc = (DLGPROC) CTrueDialog::WndProcPP;
	psp[2].pszTitle = MAKEINTRESOURCE(IDS_ABOUT);
	psp[2].lParam = (LPARAM) &m_about;
	psp[2].pfnCallback = NULL;

	RECT rcToolbar;
	GetWindowRect(hWnd, &rcToolbar);

	HWND parentWnd = CreateWindowEx(0,
								OPTIONSPP_CLASS, NULL, 
								WS_POPUP, 
								rcToolbar.left, rcToolbar.top, 
								rcToolbar.right - rcToolbar.left, 
								rcToolbar.bottom - rcToolbar.top, 
								NULL, NULL, g_hInst, (LPVOID) this);

	psh.dwSize = sizeof(PROPSHEETHEADER);
	psh.dwFlags = PSH_PROPSHEETPAGE | PSH_NOAPPLYNOW | PSH_NOCONTEXTHELP | PSH_MODELESS;
	psh.hwndParent = parentWnd;
	psh.hInstance = m_hResource;
	psh.pszIcon = NULL;
	psh.pszCaption = MAKEINTRESOURCE(IDS_NAME);
	psh.nPages = sizeof(psp) / sizeof(PROPSHEETPAGE);
	psh.nStartPage = 0;
	psh.ppsp = (LPCPROPSHEETPAGE) &psp;
	psh.pfnCallback = NULL;

	DWORD ret = NULL;

	m_hWnd = (HWND) PropertySheet(&psh);
	
	MSG msg;
	while(GetMessage(&msg, NULL, 0, 0))  
	{
		if(!PropSheet_IsDialogMessage(m_hWnd, &msg))
		{
			DispatchMessage(&msg);
		}
		if(!PropSheet_GetCurrentPageHwnd(m_hWnd))
		{
			ret = PropSheet_GetResult(m_hWnd);
			DestroyWindow(m_hWnd);
			DestroyWindow(parentWnd);
			m_hWnd = NULL;
			break;
		}
	}
	return ret;
}

void CAboutDlg::OnInitDialog(void)
{
	m_urlHome.SetWindow(GetDlgItem(m_hWnd, IDC_URLNM));
	m_urlHome.SetURL("http://www.truelaunchbar.com/plugins/netmon.php");

	m_urlTLB.SetWindow(GetDlgItem(m_hWnd, IDC_URLTLB));
	m_urlTLB.SetURL("http://www.truelaunchbar.com");

	m_urlSkins.SetWindow(GetDlgItem(m_hWnd, IDC_URLSKINS));
	m_urlSkins.SetURL("http://www.truelaunchbar.com/skinslib/index.php?show=5");

	char str[MAX_PATH];
	GetModuleFileName(g_hInst, str, MAX_PATH);
	DWORD versionLS, versionMS;
	GetFileVersion(str, versionMS, versionLS);

	char strVer[255];
	Version2Str(versionMS, versionLS, strVer);
	char strText[255];
	sprintf(strText, "v.%s", strVer);

	SetDlgItemText(m_hWnd, IDC_VERSION, strText);

	icon.LoadResource(FindResource(g_hInst, MAKEINTRESOURCE(IDR_LOGO), "PNG"), CXIMAGE_FORMAT_PNG, g_hInst);
}

void CAboutDlg::OnPaint(void)
{
	PAINTSTRUCT ps;
	BeginPaint(m_hWnd, &ps);

	RECT rc;
	GetWindowRect(GetDlgItem(m_hWnd, IDC_LOGO), &rc);
	MapWindowPoints(NULL, m_hWnd, (LPPOINT) &rc, 2);

	icon.Draw(ps.hdc, rc.left, rc.top);

	EndPaint(m_hWnd, &ps);
}

void CCustConnectionsDlg::OnInitDialog(void)
{
	HWND hList = GetDlgItem(m_hWnd, IDC_LST);
	ListView_SetExtendedListViewStyleEx(hList, LVS_EX_FULLROWSELECT | LVS_EX_INFOTIP, LVS_EX_FULLROWSELECT | LVS_EX_INFOTIP);

	RECT rcList;
	GetClientRect(hList, &rcList);

	LVCOLUMN lvc;
	ZeroMemory(&lvc, sizeof(lvc));
	lvc.mask = LVCF_WIDTH;
	lvc.cx = rcList.right - rcList.left - GetSystemMetrics(SM_CXVSCROLL);
	ListView_InsertColumn(hList, 0, &lvc);

	FillList();

	SetVis();
}

void CCustConnectionsDlg::SetVis(void)
{
	HWND hWndMsgList = GetDlgItem(m_hWnd, IDC_LST);

	int sz = ListView_GetItemCount(hWndMsgList);
	BOOL sel = FALSE;
	for(int i=0; i < sz; i++)
	{
		LVITEM lvi;
		ZeroMemory(&lvi, sizeof(lvi));
		lvi.iItem = i;
		lvi.iSubItem = 0;
		lvi.mask = LVIF_PARAM | LVIF_STATE;
		lvi.stateMask = LVIS_SELECTED;
		ListView_GetItem(hWndMsgList, &lvi);

		if(lvi.state & LVIS_SELECTED)
		{
			sel = TRUE;
			break;
		}
	}
	EnableWindow(GetDlgItem(m_hWnd, IDC_HIDE), sel);
	EnableWindow(GetDlgItem(m_hWnd, IDC_BWGROUP), sel);
	EnableWindow(GetDlgItem(m_hWnd, IDC_BWKB), sel);
	EnableWindow(GetDlgItem(m_hWnd, IDC_LINKSPEED), sel);
	EnableWindow(GetDlgItem(m_hWnd, IDC_AUTO), sel);
	EnableWindow(GetDlgItem(m_hWnd, IDC_CUSTOM), sel);
	EnableWindow(GetDlgItem(m_hWnd, IDC_BANDWIDTH), sel);
	EnableWindow(GetDlgItem(m_hWnd, IDC_DELETE), sel);
}

DWORD CCustConnectionsDlg::OnNotify(int idCtrl, LPNMHDR pnmh)
{
	switch(pnmh->code)
	{
	case PSN_APPLY:
		break;
	case PSN_HELP:
		break;
	}

	switch(idCtrl)
	{
	case IDC_LST:
		{
			LPNMLISTVIEW pnmv = (LPNMLISTVIEW) pnmh;
			if(pnmv->hdr.code == LVN_ITEMCHANGED)
			{
				if(pnmv->uNewState & LVIS_SELECTED)
				{
					FillForm(pnmv->lParam);
				}
				SetVis();
			}
		}
	}
	return TRUE;
}

CAddConnectionDlg::CAddConnectionDlg(void) : CTrueDialog(IDD_ADDCON)
{
}

CAddConnectionDlg::~CAddConnectionDlg(void)
{
}

void CCustConnectionsDlg::OnCommand(DWORD itemID)
{
	int idx = GetCurConnection();
	switch(LOWORD(itemID))
	{
	case IDC_HIDE:
		if(HIWORD(itemID) == BN_CLICKED && idx >= 0 && idx < m_parent->m_customConnections.GetCount() && !m_lockUpdate)
		{
			if(SendDlgItemMessage(m_hWnd, IDC_HIDE, BM_GETCHECK, 0, 0))
			{
				m_parent->m_customConnections[idx].flags |= CC_FLAG_HIDE;
			} else
			{
				m_parent->m_customConnections[idx].flags &= 0xF;
			}
		}
		break;
	case IDC_LINKSPEED:
	case IDC_CUSTOM:
	case IDC_AUTO:
		if(HIWORD(itemID) == BN_CLICKED && idx >= 0 && idx < m_parent->m_customConnections.GetCount() && !m_lockUpdate)
		{
			m_parent->m_customConnections[idx].flags &= 0x80;
			if(SendDlgItemMessage(m_hWnd, IDC_AUTO, BM_GETCHECK, 0, 0))
			{
				m_parent->m_customConnections[idx].flags |= CC_FLAG_BWAUTO;
			}
			if(SendDlgItemMessage(m_hWnd, IDC_LINKSPEED, BM_GETCHECK, 0, 0))
			{
				m_parent->m_customConnections[idx].flags |= CC_FLAG_BWDEFAULT;
			}
			if(SendDlgItemMessage(m_hWnd, IDC_CUSTOM, BM_GETCHECK, 0, 0))
			{
				m_parent->m_customConnections[idx].flags |= CC_FLAG_BWCUSTOM;
			}
		}
		break;
	case IDC_BANDWIDTH:
		if(HIWORD(itemID) == EN_CHANGE && idx >= 0 && idx < m_parent->m_customConnections.GetCount() && !m_lockUpdate)
		{
			m_parent->m_customConnections[idx].bandWidthLimit = GetDlgItemInt(m_hWnd, IDC_BANDWIDTH, NULL, FALSE);
		}
		break;
	case IDC_DELETE:
		{
			TCHAR msg[255];
			LoadString(m_parent->m_hResource, STR_ID_DELCONFIRM, msg, 255);
			if(MessageBox(m_hWnd, msg, "Net Monitor", MB_YESNO | MB_ICONQUESTION) == IDYES)
			{
				HWND hWndMsgList = GetDlgItem(m_hWnd, IDC_LST);

				int sz = ListView_GetItemCount(hWndMsgList);
				BOOL sel = FALSE;
				for(int i=0; i < sz; i++)
				{
					LVITEM lvi;
					ZeroMemory(&lvi, sizeof(lvi));
					lvi.iItem = i;
					lvi.mask = LVIF_STATE | LVIF_PARAM;
					lvi.stateMask = LVIS_SELECTED;
					ListView_GetItem(hWndMsgList, &lvi);
					if(lvi.state & LVIS_SELECTED)
					{
						m_parent->m_customConnections.Delete(lvi.lParam);
						FillList();
						break;
					}
				}
			}
		}
		break;
	case IDC_ADD:
		{
			CAddConnectionDlg dlg;
			if(dlg.DoModal(m_hWnd, m_parent->m_hResource) == IDOK)
			{
				int idx = -1;
				for(int i=0; i < m_parent->m_customConnections.GetCount(); i++)
				{
					if(!wcscmp (m_parent->m_customConnections[i].id, dlg.m_connection.id))
					{
						strcpy(m_parent->m_customConnections[i].displayName, dlg.m_connection.displayName);
						idx = i;
						break;
					}
				}
				if(idx < 0)
				{
					m_parent->m_customConnections.Add(&dlg.m_connection);
					AddConnection(m_parent->m_customConnections.GetCount() - 1);
				} else
				{
					HWND hWndMsgList = GetDlgItem(m_hWnd, IDC_LST);

					int sz = ListView_GetItemCount(hWndMsgList);
					BOOL sel = FALSE;
					for(int i=0; i < sz; i++)
					{
						LVITEM lvi;
						ZeroMemory(&lvi, sizeof(lvi));
						lvi.iItem = i;
						lvi.mask = LVIF_PARAM;
						ListView_GetItem(hWndMsgList, &lvi);

						if((int) lvi.lParam == idx)
						{
							ListView_SetItemText(hWndMsgList, i, 0, m_parent->m_customConnections[idx].displayName);
							ListView_SetItemState(hWndMsgList, i, LVIS_SELECTED, LVIS_SELECTED);
							break;
						}
					}
				}
			}
		}
		break;
	}
}

void CAddConnectionDlg::OnInitDialog(void)
{
	DWORD intNumber = 0;
	GetNumberOfInterfaces(&intNumber);
	ULONG szBuf = sizeof(IP_INTERFACE_INFO) * intNumber;
	PIP_INTERFACE_INFO intTable = (PIP_INTERFACE_INFO) malloc(szBuf);
	DWORD ret = GetInterfaceInfo(intTable, &szBuf);
	if(ret == ERROR_INSUFFICIENT_BUFFER)
	{
		intTable = (PIP_INTERFACE_INFO) realloc(intTable, szBuf);
		ret = GetInterfaceInfo(intTable, &szBuf);
	}
	DWORD tm = GetTickCount();
	BOOL added = FALSE;
	BOOL deleted = FALSE;
	if(ret == NO_ERROR)
	{
		MIB_IFROW ifRow;
		for(int i=0; i < intTable->NumAdapters; i++)
		{
			ifRow.dwIndex = intTable->Adapter[i].Index;
			GetIfEntry(&ifRow);
			if(ifRow.dwType == MIB_IF_TYPE_LOOPBACK) continue;
			CUSTOM_CONNECTION conn;
			conn.bandWidthLimit = 0;
			conn.flags = 0;
			memcpy(conn.id, intTable->Adapter[i].Name, min(sizeof(conn.id), sizeof(intTable->Adapter[i].Name)));
			if(!FindConnectionName(conn.id, conn.displayName, sizeof(conn.displayName)))
			{
				int sz = min(ifRow.dwDescrLen, sizeof(conn.displayName));
				strncpy(conn.displayName, (LPSTR) ifRow.bDescr, sz);
				conn.displayName[sz] = 0;
			}
			m_connections.Add(&conn);
			int idx = SendDlgItemMessage(m_hWnd, IDC_LST, LB_ADDSTRING, 0, (LPARAM) conn.displayName);
			SendDlgItemMessage(m_hWnd, IDC_LST, LB_SETITEMDATA, idx, (LPARAM) (m_connections.GetCount() - 1));
		}
	}
	free(intTable);
}

BOOL CAddConnectionDlg::OnEndDialog(void)
{
	int idx = SendDlgItemMessage(m_hWnd, IDC_LST, LB_GETCURSEL, 0, 0);
	if(idx < 0) return FALSE;
	idx = SendDlgItemMessage(m_hWnd, IDC_LST, LB_GETITEMDATA, idx, 0);
	if(idx >= 0 && idx < m_connections.GetCount())
	{
		m_connection = m_connections[idx];
		return TRUE;
	}

	return FALSE;
}

void CCustConnectionsDlg::AddConnection(int idx)
{
	HWND hList = GetDlgItem(m_hWnd, IDC_LST);
	LVITEM lvi;
	ZeroMemory(&lvi, sizeof(lvi));
	lvi.mask = LVIF_PARAM | LVIF_TEXT;
	lvi.iItem = idx;
	lvi.lParam = idx;
	lvi.pszText = m_parent->m_customConnections[idx].displayName;
	int i = ListView_InsertItem(hList, &lvi);
	ListView_SetItemState(hList, i, LVIS_SELECTED, LVIS_SELECTED);
	SetVis();
}

void CCustConnectionsDlg::FillForm(int idx)
{
	m_lockUpdate = TRUE;
	if(m_parent->m_customConnections[idx].flags & CC_FLAG_HIDE)
	{
		SendDlgItemMessage(m_hWnd, IDC_HIDE, BM_SETCHECK, TRUE, 0);
	} else
	{
		SendDlgItemMessage(m_hWnd, IDC_HIDE, BM_SETCHECK, FALSE, 0);
	}
	switch(m_parent->m_customConnections[idx].flags & 0x0F)
	{
	case CC_FLAG_BWDEFAULT:
		SendDlgItemMessage(m_hWnd, IDC_LINKSPEED, BM_SETCHECK, TRUE, 0);
		SendDlgItemMessage(m_hWnd, IDC_AUTO, BM_SETCHECK, FALSE, 0);
		SendDlgItemMessage(m_hWnd, IDC_CUSTOM, BM_SETCHECK, FALSE, 0);
		break;
	case CC_FLAG_BWAUTO:
		SendDlgItemMessage(m_hWnd, IDC_AUTO, BM_SETCHECK, TRUE, 0);
		SendDlgItemMessage(m_hWnd, IDC_LINKSPEED, BM_SETCHECK, FALSE, 0);
		SendDlgItemMessage(m_hWnd, IDC_CUSTOM, BM_SETCHECK, FALSE, 0);
		break;
	case CC_FLAG_BWCUSTOM:
		SendDlgItemMessage(m_hWnd, IDC_CUSTOM, BM_SETCHECK, TRUE, 0);
		SendDlgItemMessage(m_hWnd, IDC_AUTO, BM_SETCHECK, FALSE, 0);
		SendDlgItemMessage(m_hWnd, IDC_LINKSPEED, BM_SETCHECK, FALSE, 0);
		break;
	}
	SetDlgItemInt(m_hWnd, IDC_BANDWIDTH, m_parent->m_customConnections[idx].bandWidthLimit, FALSE);
	m_lockUpdate = FALSE;
}

int CCustConnectionsDlg::GetCurConnection(void)
{
	HWND hWndMsgList = GetDlgItem(m_hWnd, IDC_LST);

	int sz = ListView_GetItemCount(hWndMsgList);
	int idx = -1;
	for(int i=0; i < sz; i++)
	{
		LVITEM lvi;
		ZeroMemory(&lvi, sizeof(lvi));
		lvi.iItem = i;
		lvi.iSubItem = 0;
		lvi.mask = LVIF_PARAM | LVIF_STATE;
		lvi.stateMask = LVIS_SELECTED;
		ListView_GetItem(hWndMsgList, &lvi);

		if(lvi.state & LVIS_SELECTED)
		{
			idx = i;
			break;
		}
	}
	return idx;
}

void CPropertiesPP::Close(void)
{
	if(m_hWnd)
	{
		DestroyWindow(m_hWnd);
		m_hWnd = NULL;
	}
}

void CCustConnectionsDlg::FillList(void)
{
	HWND hList = GetDlgItem(m_hWnd, IDC_LST);

	ListView_DeleteAllItems(hList);

	for(int i=0; i < m_parent->m_customConnections.GetCount(); i++)
	{
		LVITEM lvi;
		ZeroMemory(&lvi, sizeof(lvi));
		lvi.mask = LVIF_PARAM | LVIF_TEXT;
		lvi.iItem = i;
		lvi.lParam = i;
		lvi.pszText = m_parent->m_customConnections[i].displayName;
		ListView_InsertItem(hList, &lvi);
	}
}
