// StaticLink.cpp: implementation of the CStaticLink class.
//
//////////////////////////////////////////////////////////////////////

#define _WIN32_WINNT  0x0400
#include "globals.h"
#include "StaticLink.h"
#include <commctrl.h>

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CStaticLink::CStaticLink()
{
	m_hWnd = NULL;
	oldWndProc = NULL;
	m_hFont = NULL;
	m_hCursor = LoadCursor(NULL, IDC_HAND);
	m_url[0] = 0;
	m_bTrackLeave = FALSE;
	m_hFontOver = NULL;
}

CStaticLink::~CStaticLink()
{
	if(m_hCursor) DeleteObject(m_hCursor);
	if(m_hFont) DeleteObject(m_hFont);
	if(m_hFontOver) DeleteObject(m_hFontOver);
}

void CStaticLink::SetWindow(HWND hWnd)
{
	m_hWnd = hWnd;
	SetProp(hWnd, TEXT("This"), (HANDLE) this);
	SetProp(hWnd, TEXT("StaticLink"), (HANDLE) 0x55AA55AA);
	oldWndProc = (WNDPROC) SetWindowLong(hWnd, GWL_WNDPROC, (LONG) CStaticLink::WndProc);
}

LRESULT CALLBACK CStaticLink::WndProc(HWND hWnd, UINT uMessage, WPARAM wParam, LPARAM lParam)
{
	CStaticLink* pThis = NULL;
	if(IsWindow(hWnd))
	{
		pThis = (CStaticLink*)GetProp(hWnd, TEXT("This"));
	}

	MSG msg;
	memset(&msg, 0, sizeof(msg));
	msg.hwnd = hWnd;
	msg.lParam = lParam;
	msg.message = uMessage;
	msg.wParam = wParam;
	LRESULT res = NULL;

	if(pThis)
	{
		switch (uMessage)
		{
		case WM_LBUTTONDOWN:
			pThis->OnClick();
			break;
		case WM_CTLCOLORSTATIC:
			return pThis->OnCtlColor((HDC) wParam);
		case WM_NCHITTEST:
			return HTCLIENT;
		case WM_SETCURSOR:
			return pThis->OnSetCursor();
		case WM_MOUSEMOVE:
			{
				if(!pThis->m_bTrackLeave)
				{
					TRACKMOUSEEVENT tme;
					tme.cbSize = sizeof(tme);
					tme.hwndTrack = pThis->m_hWnd;
					tme.dwFlags = TME_LEAVE;
					pThis->m_bTrackLeave = TRUE;
					_TrackMouseEvent(&tme);
					InvalidateRect(pThis->m_hWnd, NULL, TRUE);
				}
			}
			break;
		case WM_MOUSELEAVE:
			{
				pThis->m_bTrackLeave = FALSE;
				InvalidateRect(pThis->m_hWnd, NULL, TRUE);
			}
			break;
		}
	}

	return CallWindowProc(pThis->oldWndProc, hWnd, uMessage, wParam, lParam);
}

LRESULT CStaticLink::OnCtlColor(HDC dc)
{
	HBRUSH hbr = NULL;
	if(!m_hFont || !m_hFontOver)
	{
		LOGFONT lf;
		HFONT fnt = (HFONT) SendMessage(m_hWnd, WM_GETFONT, 0, 0);
		GetObject(fnt, sizeof(lf), &lf);
		if(!m_hFont)
			m_hFont = CreateFontIndirect(&lf);
		lf.lfUnderline = TRUE;
		if(!m_hFontOver)
			m_hFontOver = CreateFontIndirect(&lf);
	}
	
	// use underline font and visited/unvisited colors
	if(m_bTrackLeave)
	{
		SelectObject(dc, m_hFontOver);
	} else
	{
		SelectObject(dc, m_hFont);
	}
	//SendMessage(GetParent(m_hWnd), WM_ERASEBKGND, (WPARAM) dc, NULL);
	SetTextColor(dc, RGB(0,0,255));
	SetBkMode(dc, TRANSPARENT);

	// return hollow brush to preserve parent background color
	//hbr = (HBRUSH)::GetStockObject(HOLLOW_BRUSH);

	//return (LRESULT) hbr;
	return (LRESULT)::GetStockObject(HOLLOW_BRUSH);;
}

LRESULT CStaticLink::OnSetCursor()
{
	if(m_hCursor)
	{
		SetCursor(m_hCursor);
		return TRUE;
	}
	return FALSE;
}

void CStaticLink::SetURL(LPTSTR url)
{
	lstrcpy(m_url, url);
}

void CStaticLink::OnClick()
{
	if(m_url[0])
	{
		ShellExecute(NULL, NULL, m_url, NULL, NULL, SW_NORMAL);
	}
}
